/***************************************************************************
    libBIOS - A portable library for developing an OpenSource BIOS
    Copyright (C) 2001 - David Aparicio

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Author can be reached at 'daparic@terra.es'
****************************************************************************/
// 
#ifndef _HW_H
#define _HW_H

#define DC21285_SDRAM_CTRL		0x40000000
#define DC21285_PCI_IACK		0x79000000
#define DC21285_ARMCSR_BASE		0x42000000
#define DC21285_PCICFG0			0x7b000000
#define DC21285_PCICFG1			0x7a000000
#define DC21285_OUTBOUND_WRITE_FLUSH	0x78000000
#define DC21285_FLASH			0x41000000
#define DC21285_PCI_IO			0x7c000000
#define DC21285_PCI_MEM			0x80000000

// El mapa PCI se construye:
#undef  ARMCSR_BASE	// se define en Linux...
#define ARMCSR_BASE		DC21285_ARMCSR_BASE

#ifdef __ASSEMBLER__
#  define DC21285_IO(x)		(ARMCSR_BASE+(x))
#  define DC21285_IO16(x)	(ARMCSR_BASE+(x))
#  define DC21285_IO8(x)	(ARMCSR_BASE+(x))
#else
#  define DC21285_IO(x)		((volatile unsigned long  *)(ARMCSR_BASE+(x)))
#  define DC21285_IO16(x)	((volatile unsigned short *)(ARMCSR_BASE+(x)))
#  define DC21285_IO8(x)	((volatile unsigned char  *)(ARMCSR_BASE+(x)))
#  define PCIMEM(x)	((volatile unsigned char *)(DC21285_PCI_MEM+(x)))
#  define PCIMEM16(x)	((volatile unsigned short *)(DC21285_PCI_MEM+(x)))
#  define PCIMEM32(x)	((volatile unsigned long *)(DC21285_PCI_MEM+(x)))
#  define PCIPORT(x)	((volatile unsigned char *)(DC21285_PCI_IO+(x)))
#  define PCIPORT16(x)	((volatile unsigned short *)(DC21285_PCI_IO+(x)))
#  define PCIPORT32(x)	((volatile unsigned long  *)(DC21285_PCI_IO+(x)))
#endif


/* CSR configuration space */
#define CSR_PCICMD		DC21285_IO16(0x0004)
#define CSR_CLASSREV		DC21285_IO(0x0008)
#define CSR_PCICACHELINESIZE	DC21285_IO8(0x000c)
#define CSR_PCICSRBASE		DC21285_IO(0x0010)
#define CSR_PCICSRIOBASE	DC21285_IO(0x0014)
#define CSR_PCISDRAMBASE	DC21285_IO(0x0018)
#define CSR_PCIROMBASE		DC21285_IO(0x0030)	// errata!!??
/* PCI control & status space */
#define CSR_PCI_INTSTATUS	DC21285_IO(0x0030)
#define CSR_PCI_INTMASK		DC21285_IO(0x0034)
#define CSR_PCI_INTLINE		DC21285_IO(0x003C)
#define		OFS_MBOX0		0x0050
#define CSR_MBOX0               DC21285_IO(OFS_MBOX0)
#define		OFS_MBOX1		0x0054
#define CSR_MBOX1               DC21285_IO(OFS_MBOX1)
#define		OFS_MBOX2		0x0058
#define CSR_MBOX2               DC21285_IO(OFS_MBOX2)
#define		OFS_MBOX3		0x005c
#define CSR_MBOX3               DC21285_IO(OFS_MBOX3)
#define CSR_DOORBELL            DC21285_IO(0x0060)
#define CSR_DOORBELL_SETUP      DC21285_IO(0x0064)
#define CSR_ROMWRITEREG		DC21285_IO(0x0068)
/* 285 control & status space */
#define CSR_DMA0_COUNT		DC21285_IO(0x0080)
#define CSR_DMA1_COUNT		DC21285_IO(0x00A0)
#define		DMA_NBYTES(A)	((u32)(A)&0x00FFFFFF)
#define		DMA_BURSTDELAY(A)	(((A)&0x3F)<<24)
#define		DMA_RAM2PCI	(1<<30)
#define		DMA_EOC		(1<<31)
				// Siempre transferencias PCI->RAM sin esperas
#define		DMA_COUNT(A)	(DMA_NBYTES(A)|DMA_BURSTDELAY(0))
#define CSR_DMA0_PCIBASE	DC21285_IO(0x0084)
#define CSR_DMA1_PCIBASE	DC21285_IO(0x00A4)
#define		DMA_PCIADDR(A)	(A)
#define CSR_DMA0_RAMBASE	DC21285_IO(0x0088)
#define CSR_DMA1_RAMBASE	DC21285_IO(0x00A8)
#define		DMA_RAMADDR(A)	((u32)(A)&0x0FFFFFFF)
#define CSR_DMA0_NEXT		DC21285_IO(0x008C)
#define CSR_DMA1_NEXT		DC21285_IO(0x00AC)
#define		DMA_NEXT(A)	((u32)(A)&0x0FFFFFF0)
#define CSR_DMA0_CONTROL	DC21285_IO(0x0090)
#define CSR_DMA1_CONTROL	DC21285_IO(0x00B0)
				// SDRAM burst 8 Dword (bits 18:16)
				// PCI   burst 8 Dword (bits 18:16)
				// SCALE delay (110/PCI) (bits 9:8)
				// PCI-read mode (bits 6:5)
#define		DMA_STOP	((3<<16)|(0<<15)|(0<<8)|(1<<7)|(1<<5)|(3<<2))
#define		DMA_START	((3<<16)|(0<<15)|(0<<8)|(1<<7)|(1<<5)|(3<<2)|1)
#define		DMA_ST_CDONE	0x80	// Se ha completado toda la cadena
#define		DMA_ST_ERROR	0x08	// Error en la transferencia
#define		DMA_ST_TDONE	0x04	// Se ha completado algun descriptor
#define		DMA_ST_ENABLED	0x01	// El canal esta activo
#define		DMA_STATUS(A)	(A&(DMA_ST_CDONE|DMA_ST_TDONE|DMA_ST_ERROR))

#define CSR_CSRBASEMASK		DC21285_IO(0x00f8)
#define CSR_CSRBASEOFFSET	DC21285_IO(0x00fc)
#define CSR_SDRAMBASEMASK	DC21285_IO(0x0100)
#define CSR_SDRAMBASEOFFSET	DC21285_IO(0x0104)
#define CSR_ROMBASEMASK		DC21285_IO(0x0108)
#define		OFS_SDRAMTIMING		0x010c
#define CSR_SDRAMTIMING		DC21285_IO(OFS_SDRAMTIMING)
#define		OFS_SDRAMADDRSIZE0	0x0110
#define CSR_SDRAMADDRSIZE0	DC21285_IO(OFS_SDRAMADDRSIZE0)
#define CSR_SDRAMADDRSIZE1	DC21285_IO(0x0114)
#define CSR_SDRAMADDRSIZE2	DC21285_IO(0x0118)
#define CSR_SDRAMADDRSIZE3	DC21285_IO(0x011c)
#define CSR_I2O_INFREEHEAD	DC21285_IO(0x0120)
#define CSR_I2O_INPOSTTAIL	DC21285_IO(0x0124)
#define CSR_I2O_OUTPOSTHEAD	DC21285_IO(0x0128)
#define CSR_I2O_OUTFREETAIL	DC21285_IO(0x012c)
#define CSR_I2O_INFREECOUNT	DC21285_IO(0x0130)
#define CSR_I2O_OUTPOSTCOUNT	DC21285_IO(0x0134)
#define CSR_I2O_INPOSTCOUNT	DC21285_IO(0x0138)
#define		OFS_SA110_CNTL		0x013c
#define CSR_SA110_CNTL		DC21285_IO(OFS_SA110_CNTL)
#define SA110_CNTL_INITCMPLETE		(1 << 0)
#define SA110_CNTL_ASSERTSERR		(1 << 1)
#define SA110_CNTL_RXSERR		(1 << 3)
#define SA110_CNTL_SA110DRAMPARITY	(1 << 4)
#define SA110_CNTL_PCISDRAMPARITY	(1 << 5)
#define SA110_CNTL_DMASDRAMPARITY	(1 << 6)
#define SA110_CNTL_DISCARDTIMER		(1 << 8)
#define SA110_CNTL_PCINRESET		(1 << 9)
#define SA110_CNTL_I2O_256		(0 << 10)
#define SA110_CNTL_I20_512		(1 << 10)
#define SA110_CNTL_I2O_1024		(2 << 10)
#define SA110_CNTL_I2O_2048		(3 << 10)
#define SA110_CNTL_I2O_4096		(4 << 10)
#define SA110_CNTL_I2O_8192		(5 << 10)
#define SA110_CNTL_I2O_16384		(6 << 10)
#define SA110_CNTL_I2O_32768		(7 << 10)
#define SA110_CNTL_WATCHDOG		(1 << 13)
#define SA110_CNTL_ROMWIDTH_UNDEF	(0 << 14)
#define SA110_CNTL_ROMWIDTH_16		(1 << 14)
#define SA110_CNTL_ROMWIDTH_32		(2 << 14)
#define SA110_CNTL_ROMWIDTH_8		(3 << 14)
#define SA110_CNTL_ROMACCESSTIME(x)	((x)<<16)
#define SA110_CNTL_ROMBURSTTIME(x)	((x)<<20)
#define SA110_CNTL_ROMTRISTATETIME(x)	((x)<<24)
#define SA110_CNTL_XCSDIR(x)		((x)<<28)
#define SA110_CNTL_PCICFN		(1 << 31)

#define CSR_SDRAM0_CTRL		DC21285_IO(0x0110)
#define CSR_SDRAM1_CTRL		DC21285_IO(0x0114)
#define CSR_SDRAM2_CTRL		DC21285_IO(0x0118)
#define CSR_SDRAM3_CTRL		DC21285_IO(0x011C)

#define CSR_PCIADDR_EXTN	DC21285_IO(0x0140)
#define CSR_PREFETCHMEMRANGE	DC21285_IO(0x0144)
#define CSR_XBUS_CYCLE		DC21285_IO(0x0148)
#define CSR_XBUS_IOSTROBE	DC21285_IO(0x014c)
#define CSR_DOORBELL_PCI	DC21285_IO(0x0150)
#define CSR_DOORBELL_SA110	DC21285_IO(0x0154)
#define		OFS_UARTDR		0x0160
#define CSR_UARTDR		DC21285_IO(OFS_UARTDR)
#define		OFS_RXSTAT		0x0164
#define CSR_RXSTAT		DC21285_IO(OFS_RXSTAT)
#define		OFS_H_UBRLCR		0x0168
#define CSR_H_UBRLCR		DC21285_IO(OFS_H_UBRLCR)
#define		OFS_M_UBRLCR		0x016c
#define CSR_M_UBRLCR		DC21285_IO(OFS_M_UBRLCR)
#define		OFS_L_UBRLCR		0x0170
#define CSR_L_UBRLCR		DC21285_IO(OFS_L_UBRLCR)
#define		OFS_UARTCON		0x0174
#define CSR_UARTCON		DC21285_IO(OFS_UARTCON)
#define		OFS_UARTFLG		0x0178
#define CSR_UARTFLG		DC21285_IO(OFS_UARTFLG)
#define		FLG_UART_TXCHARSLEFT	(1<<3)
#define		FLG_UART_RFIFOEMPTY	(1<<4)
#define		FLG_UART_TFIFOFULL	(1<<5)
#define 	OFS_IRQ_STATUS		0x0180
#define CSR_IRQ_STATUS		DC21285_IO(OFS_IRQ_STATUS)
#define 	OFS_IRQ_RAWSTATUS	0x0184
#define CSR_IRQ_RAWSTATUS	DC21285_IO(OFS_IRQ_RAWSTATUS)
#define 	OFS_IRQ_ENABLE		0x0188
#define CSR_IRQ_ENABLE		DC21285_IO(OFS_IRQ_ENABLE)
#define 	OFS_IRQ_DISABLE		0x018c
#define CSR_IRQ_DISABLE		DC21285_IO(OFS_IRQ_DISABLE)
#define 	OFS_IRQ_SOFT		0x0190
#define CSR_IRQ_SOFT		DC21285_IO(OFS_IRQ_SOFT)
#define		FLG_IRQ_SOFT		(1<<1)
#define		FLG_IRQ_RX		(1<<2)
#define		FLG_IRQ_TX		(1<<3)
#define		FLG_IRQ_TIMER1		(1<<4)
#define		FLG_IRQ_TIMER2		(1<<5)
#define		FLG_IRQ_TIMER3		(1<<6)
#define		FLG_IRQ_TIMER4		(1<<7)
#define		FLG_IRQ_IN0		(1<<8)
#define		FLG_IRQ_IN1		(1<<9)
#define		FLG_IRQ_IN2		(1<<10)
#define		FLG_IRQ_IN3		(1<<11)
#define		FLG_IRQ_XCS0		(1<<12)
#define		FLG_IRQ_XCS1		(1<<13)
#define		FLG_IRQ_XCS2		(1<<14)
#define		FLG_IRQ_DOORBELL	(1<<15)
#define CSR_FIQ_STATUS		DC21285_IO(0x0280)
#define CSR_FIQ_RAWSTATUS	DC21285_IO(0x0284)
#define 	OFS_FIQ_ENABLE		0x0288
#define CSR_FIQ_ENABLE		DC21285_IO(OFS_FIQ_ENABLE)
#define 	OFS_FIQ_DISABLE		0x028c
#define CSR_FIQ_DISABLE		DC21285_IO(OFS_FIQ_DISABLE)
#define CSR_FIQ_SOFT		DC21285_IO(0x0290)
#define		OFS_TIMER1_LOAD		0x0300
#define CSR_TIMER1_LOAD		DC21285_IO(OFS_TIMER1_LOAD)
#define		OFS_TIMER1_VALUE	0x0304
#define CSR_TIMER1_VALUE	DC21285_IO(OFS_TIMER1_VALUE)
#define		OFS_TIMER1_CNTL		0x0308
#define CSR_TIMER1_CNTL		DC21285_IO(OFS_TIMER1_CNTL)
#define		OFS_TIMER1_CLR		0x030c
#define CSR_TIMER1_CLR		DC21285_IO(OFS_TIMER1_CLR)
#define		OFS_TIMER2_LOAD		0x0320
#define CSR_TIMER2_LOAD		DC21285_IO(OFS_TIMER2_LOAD)
#define		OFS_TIMER2_VALUE	0x0324
#define CSR_TIMER2_VALUE	DC21285_IO(OFS_TIMER2_VALUE)
#define		OFS_TIMER2_CNTL		0x0328
#define CSR_TIMER2_CNTL		DC21285_IO(OFS_TIMER2_CNTL)
#define		OFS_TIMER2_CLR		0x032c
#define CSR_TIMER2_CLR		DC21285_IO(OFS_TIMER2_CLR)

#define		OFS_TIMER3_LOAD		0x0340
#define CSR_TIMER3_LOAD		DC21285_IO(OFS_TIMER3_LOAD)
#define		OFS_TIMER3_VALUE	0x0344
#define CSR_TIMER3_VALUE	DC21285_IO(OFS_TIMER3_VALUE)
#define		OFS_TIMER3_CNTL		0x0348
#define CSR_TIMER3_CNTL		DC21285_IO(OFS_TIMER3_CNTL)
#define		OFS_TIMER3_CLR		0x034c
#define CSR_TIMER3_CLR		DC21285_IO(OFS_TIMER3_CLR)

#define CSR_TIMER4_LOAD		DC21285_IO(0x0360)
#define CSR_TIMER4_VALUE	DC21285_IO(0x0364)
#define CSR_TIMER4_CNTL		DC21285_IO(0x0368)
#define CSR_TIMER4_CLR		DC21285_IO(0x036c)

#define TIMER_CNTL_DISABLE	(0)
#define TIMER_CNTL_ENABLE	(1 << 7)
#define TIMER_CNTL_AUTORELOAD	(1 << 6)
#define TIMER_CNTL_DIV1		(0)
#define TIMER_CNTL_DIV16	(1 << 2)
#define TIMER_CNTL_DIV256	(2 << 2)
#define TIMER_CNTL_CNTEXT	(3 << 2)

//-------------------SA110-------------------
#define csrIRQf	0x80
#define csrFIQf	0x40
#define csrMMMm	0x1F
#define csrSVCm	0x13
#define csrIRQm	0x12
#define csrFIQm	0x11
#define csrUSRm	0x10
#define csrER1m	0x17	// Abort
#define csrER2m	0x1B	// Undef

#endif	// _HW_H
