/***************************************************************************
    libBIOS - A portable library for developing an OpenSource BIOS
    Copyright (C) 2001 - David Aparicio

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Author can be reached at 'daparic@terra.es'
****************************************************************************/
// 
#ifndef __LIBBIOS_H
#define __LIBBIOS_H

#define MAXCSR		64	// Max number of cards in your system...
#define	LEN_LABEL	32	// Max length of filename in image header.
#define MAX_FILES	16	// Number of files in a LOADER

//--- useful numbers for memory map calculations...
#define SZ_1K                           0x00000400
#define SZ_4K                           0x00001000
#define SZ_16K                          0x00004000
#define SZ_32K                          0x00008000
#define SZ_48K                          0x0000C000
#define SZ_64K                          0x00010000
#define SZ_128K                         0x00020000
#define SZ_256K                         0x00040000
#define SZ_512K                         0x00080000

#define SZ_1M                           0x00100000
#define SZ_2M                           0x00200000
#define SZ_3M                           0x00300000
#define SZ_4M                           0x00400000
#define SZ_8M                           0x00800000
#define SZ_16M                          0x01000000
#define SZ_32M                          0x02000000
#define SZ_64M                          0x04000000
#define SZ_128M                         0x08000000
#define SZ_256M                         0x10000000
#define SZ_512M                         0x20000000
#define SZ_1G                           0x40000000
#define SZ_2G                           0x80000000

// Device identifiers. This will recognice CPU parts in order to load images
//--- identifiers for PCI-located CPUs
#define ID_EBSA285	0x10111065
#define	ID_DSPMCA2	0x12D51000

// Memory map (virtual) (64MB)
//--- This is related to a SA110/285 native map
//--- I would like to hear from SA1100 and i310 guys...
#define	ADDR_CODE	(0x00000000      )	// SIZE= 01MB
#define	ADDR_DATA	(0x00000000+SZ_1M)	// SIZE= 03MB
#define	ADDR_LOAD	(0x00000000+SZ_4M)	// SIZE= 04MB
#define	ADDR_TEMP	(0x00000000+SZ_8M)	// SIZE= 08MB
#define	ADDR_MCB	(0x00000000+SZ_16M)	// SIZE= (total - 16MB) = 48MB
#define ADDR_POST	(0x41000000)
#define ADDR_ROMLOAD	(ADDR_POST +SZ_16K)	// LOADER in FLASH?

#define INLINE static __inline__


// --- ctype.h, "stolen" from newlib...
#define	_ANSIDECL_H_
#define __SYS_CONFIG_H__
#define _HAVE_STDC

#define PCI_DEVICE(CSR)	((CSR)->vendor<<16|(CSR)->device)
#define PCI_CLASS(CSR)  ((CSR)->class1<<8|(CSR)->class2)
#define PCI_CLASS_SCSI		0x0100
#define PCI_CLASS_IDE		0x0101
#define PCI_CLASS_ETH		0x0200
#define PCI_CLASS_BRIDGE_ISAPCI	0x0601
#define PCI_CLASS_BRIDGE_PCIPCI	0x0604
#define PCI_CLASS_CPU		0x0B40

#define LIBENTRY	// Marker for PUBLIC LIB entries.

#ifndef	__USE_GLIBC__
#ifndef NULL
#define NULL ((void *)0)
#endif

#define _POINTER_INT long
#define	_PTR		void *
#define	_AND		,
#define	_NOARGS		void
#define	_CONST		const
#define	_VOLATILE	volatile
#define	_SIGNED		signed
#define	_DOTS		, ...
#define _VOID void
#define	_EXFUN(name, proto)		name proto
#define	_DEFUN(name, arglist, args)	name(args)
#define	_DEFUN_VOID(name)		name(_NOARGS)
#define _CAST_VOID (void)
#define _LONG_DOUBLE long double
#define _PARAMS(paramlist)		paramlist
#endif	// _USE_GLIBC_

#define SFS_NOPCI(A)	((A&0xFFFF0000)==0xFFFF0000)
#define SFS_ARCH_SA110	0x00000028
#define SFS_ARCH_DSP	0x0000E817
#define SFS_ARCH_LCA	0xFFFF0000

#define SFS_TYPE_POST	0x0DAC0001
#define SFS_TYPE_LOAD	0x0DAC0002
#define SFS_TYPE_EXEC	0x0DAC0003
#define SFS_TYPE_BZIP	0x0DAC0004	// Compressed EXEC

#ifndef __ASSEMBLER__
#include <stddef.h>	/* size_t et al */
#include <stdarg.h>	/* va_arg is a compiler-dependant stuff */

// --- --- --- --- basic types --- --- ---
typedef unsigned long  u32;
typedef unsigned short u16;
typedef unsigned char  u8;
typedef   signed long  s32;
typedef   signed short s16;
typedef   signed char  s8;
typedef const unsigned long  cu32;
typedef const unsigned short cu16;
typedef const unsigned char  cu8;
typedef const   signed long  cs32;
typedef const   signed short cs16;
typedef const   signed char  cs8;

//--- macros for distinguish binary types
#define IMG_POST(A)	((*A)[0].type==SFS_TYPE_POST)
#define IMG_LOAD(A)	((*A)[0].type==SFS_TYPE_LOAD)
#define IMG_EXEC(A)	(!IMG_POST(A) && !IMG_LOAD(A))

// POST: Mailbox with environment info for all programs
typedef struct {
	u32 ramsize;
	u32 romsize;
	u32 warmreset;		// first time 2nd step is run, put nonzero here
	} env_t;

// ROM: Entry for every "block" (in "LOAD" & "POST")
typedef struct {
	//--all binaries (mbF_t[0])
	u32 offset; 		// if(!LOADER) Start Branch (ARM code)
				// if( LOADER && mbF_t[x]) byte offset in image
	u32 arch;
	u32 type;		// id of target-device for this binary
				// special value for mbF_t[0]
	//--only for "LOADER"
	u32 entry;		// entry point (start of execution)
	u8  label[LEN_LABEL];	// name of this file (32 bytes)
	u8 *baseC;
	u32 sizeC;	// CODE segment base and size
	u8 *baseD;
	u32 sizeD;	// DATA segment base and size
	} mbE_t;	// Multi-block Entry

// Multi-block Array
typedef mbE_t mbA_t[MAX_FILES];

#ifndef	__USE_GLIBC__
/* Support gcc's __attribute__ facility.  */
#define _ATTRIBUTE(attrs) __attribute__ (attrs)

#define	_U	01
#define	_L	02
#define	_N	04
#define	_S	010
#define _P	020
#define _C	040
#define _X	0100
#define	_B	0200

extern	_CONST char	_ctype_[];

#define	isalpha(c)	((_ctype_+1)[(unsigned)(c)]&(_U|_L))
#define	isupper(c)	((_ctype_+1)[(unsigned)(c)]&_U)
#define	islower(c)	((_ctype_+1)[(unsigned)(c)]&_L)
#define	isdigit(c)	((_ctype_+1)[(unsigned)(c)]&_N)
#define	isxdigit(c)	((_ctype_+1)[(unsigned)(c)]&(_X|_N))
#define	isspace(c)	((_ctype_+1)[(unsigned)(c)]&_S)
#define ispunct(c)	((_ctype_+1)[(unsigned)(c)]&_P)
#define isalnum(c)	((_ctype_+1)[(unsigned)(c)]&(_U|_L|_N))
#define isprint(c)	((_ctype_+1)[(unsigned)(c)]&(_P|_U|_L|_N|_B))
#define	isgraph(c)	((_ctype_+1)[(unsigned)(c)]&(_P|_U|_L|_N))
#define iscntrl(c)	((_ctype_+1)[(unsigned)(c)]&_C)
#define toupper(c)	(islower(c) ? (c - 'a' + 'A') : c)
#define tolower(c)	(isupper(c) ? (c - 'A' + 'a') : c)

#define isascii(c)	((unsigned)(c)<=0177)
#define toascii(c)	((c)&0177)

typedef long int __int32_t;
typedef unsigned long int __uint32_t;

#endif	//	__USE_GLIBC__

// --- PCI basic sizes
#define PCI_BUSMAX 128
#define PCI_DEVMAX 21
#define PCI_FUNMAX 8
#define PCI_GRPMAX 5	// resource type (0:I/O, 1:ISA, 2:ROM+MEM, 3:RAM)
#define  PCI_GRPIO  0	// PCI-I/O
#define  PCI_GRPISA 1	// PCI-MEM (low memory 1Mbyte: forbidden ouside BUS0)
#define  PCI_GRPROM 2	// PCI-MEM writable (FLASH), not prefetchable
#define  PCI_GRPMEM 2	// PCI-MEM not-prefetchable, usually mapped I/O
#define  PCI_GRPRAM 3	// PCI-MEM prefetchable, usually SDRAM
#define PCI_RESMAX 8	// max resources per type (maxnum of baseaddr in CSR)

#define PCI_CSRHDR0(C) ((C->header&0x7F)==0)
#define PCI_CSRHDR1(C) ((C->header&0x7F)==1)
#define PCI_MULTIFN(C) (C->header&0x80)

#define CSRNUL	(csr_t *)0

typedef struct {
	u32 baseaddr[6];
	u32 cis_ptr;
	u16 _vendor;
	u16 _subsystem;
	u32 rom_base;
	u32 dummy4;
	u32 dummy5;
	u8  irq, ipin, min_gnt, max_lat;
	} csrHdr0_t;

typedef struct {
	u32 baseaddr[2];
	u8  busE, busI, busmax, latency2;
	u8  iobase, iosize;
	u16 status2;
	u16 membase, memsize;	// Non-prefetchable
	u16 rambase, ramsize;	// Prefetchable
	u32 hi32_rambase;
	u32 hi32_ramsize;
	u16 hi16_iobase, hi16_iosize;
	u32 rom_base;
	u8  irq, ipin;
	u16 control;
	} csrHdr1_t;	// Puente PCI/PCI

typedef struct {
	// para ioctl, "vendor" se usa como entrada de indice PCI
	u16 vendor;
	u16 device;
	u16 command;
	u16 status;
	u8  revID, pritf, class2, class1;
	u8  cache_size, latency, header, bist;

	union {
		csrHdr0_t	hdr0;
		csrHdr1_t	hdr1;
		} u;
	} csr_t;

// Generic geometry for PCI bus
//	c: Card index in detected devices array
//	t: Type of resource (0: I/O, 1: ISA, 2: ROM, 3:MEM, 4: RAM)
//	r: Resource index for resource and card (0..5)

typedef struct {
	csr_t  *csr;
	u32    *addr[PCI_GRPMAX][PCI_RESMAX];
	u32	size[PCI_GRPMAX][PCI_RESMAX];
	} pcigrp_t;

typedef pcigrp_t csrGrp_t[MAXCSR];

typedef struct {
	u32 min[PCI_GRPMAX];
	u32 max[PCI_GRPMAX];
	} pciFrame_t;

LIBENTRY u32	pci_isHost(void);

//--- PCI memory mapped I/O accesing (READ/WRITE)
LIBENTRY u8     pci_rm8 (u32 port);
LIBENTRY u16    pci_rm16(u32 port);
LIBENTRY u32    pci_rm32(u32 port);

LIBENTRY void   pci_wm8 (u32 port,  u8 val);
LIBENTRY void   pci_wm16(u32 port, u16 val);
LIBENTRY void   pci_wm32(u32 port, u32 val);

//--- PCI port I/O accesing (READ/WRITE)
LIBENTRY u8     pci_rp8 (u32 port);
LIBENTRY u16    pci_rp16(u32 port);
LIBENTRY u32    pci_rp32(u32 port);

LIBENTRY void   pci_wp8 (u32 port,  u8 val);
LIBENTRY void   pci_wp16(u32 port, u16 val);
LIBENTRY void   pci_wp32(u32 port, u32 val);

//--- Mapping from nortbridge-mapping to PCI-addressing and viceversa
LIBENTRY u32    pci_bus2cpu(u32 bus_addr);
LIBENTRY u32    pci_cpu2bus(u32 cpu_addr);
//--- Mapping from SDRAM to PCI-addressing and viceversa
LIBENTRY u32    pci_bus2ram(u32 bus_addr);
LIBENTRY u32    pci_ram2bus(u32 ram_addr);

//--- Mapping from BUS/SLOT/FUNC to CSR-addressing and viceversa
LIBENTRY int    pci_cnvCsrPar (csr_t  *csr, u8 *bus, u8 *slot, u8 *fun);
LIBENTRY int    pci_cnvParCsr (u8  bus, u8  slot, u8 fun, csr_t **csr);

//--- Mechanism for initializing resources (POST/loader)
LIBENTRY int       pci_grpInit(csrGrp_t grp);
LIBENTRY void	   pci_grpAdd (csrGrp_t grp, csr_t *csr);
LIBENTRY u32	   pci_getType(volatile u32 *addr);
LIBENTRY char     *pci_strType(volatile u32 *addr);
LIBENTRY u32	   pci_getAddr(volatile u32 *addr);
LIBENTRY u32	   pci_getSize(volatile u32 *addr);
LIBENTRY int       pci_getNumSlot  (csr_t *csr);

LIBENTRY int       pci_hostInit(pciFrame_t *pfm);
LIBENTRY int       pci_devScan(csr_t *csr);
LIBENTRY void      pci_devInit(csr_t *csr, u32 addr[], u32 low[], u32 hig[]);

//--- Mechanism for scanning resources (user programs)
LIBENTRY int	   pci_scan      (u32 ramsize, u32 romsize);
LIBENTRY void      pci_grpScan   (csrGrp_t grp);
LIBENTRY pcigrp_t *pci_grpGetDev (csrGrp_t grp, int c);
LIBENTRY u32      *pci_grpGetAddr(csrGrp_t grp, int c, int t, int r);
LIBENTRY u32	   pci_grpGetSize(csrGrp_t grp, int c, int t, int r);

LIBENTRY csr_t    *pci_grpFindIndex(csrGrp_t csrg, int i);
LIBENTRY csr_t    *pci_grpFindClass(csrGrp_t grp, u16 classpair);
LIBENTRY csr_t    *pci_grpFindDevice(csrGrp_t grp, u16 vendor, u16 device);
LIBENTRY int       pci_getStrClass(csr_t *csr, char *str);
LIBENTRY int       pci_getStrDevice(csr_t *csr, char *str);

LIBENTRY u32    crc32(u8 *addr, u32 size);

#ifndef __USE_GLIBC__
//--- Typical libc routines. Please take a look at README file.
LIBENTRY u16	htons(u16 u);
LIBENTRY u32	htonl(u32 u);
LIBENTRY void 	IniHeap(unsigned frp, unsigned sizeheap);
LIBENTRY void  *malloc( unsigned long nbytes );
LIBENTRY void	free (void *ap);
LIBENTRY char  *memchr(const void *src_void, int c, unsigned long length);
LIBENTRY int	memcmp(const void *m1, const void *m2, size_t n);

LIBENTRY int	strcasecmp(const char *s1, const char *s2);
LIBENTRY char  *strcat(char *s1, const char *s2);
LIBENTRY char  *strchr(const char *s1, int i);
LIBENTRY int	strcmp(const char *s1, const char *s2);
LIBENTRY u32	strcspn(const char *s1, const char *s2);
LIBENTRY const char *strerror(int errnum);
LIBENTRY u32	strlen(const char *str);
LIBENTRY char  *strlwr (char *a);
LIBENTRY int	strncasecmp(const char *s1, const char *s2, unsigned long n);
LIBENTRY char  *strncat(char *s1, const char *s2, unsigned long n);
LIBENTRY int	strncmp(const char *s1, const char *s2, unsigned long n);
LIBENTRY char  *strncpy(char *dst0, const char *src0, unsigned long count);
LIBENTRY char  *strpbrk(const char *s1, const char *s2);
LIBENTRY char  *strrchr(const char *s, int i);
LIBENTRY u32	strspn(const char *s1, const char *s2);
LIBENTRY char  *strstr(const char *searchee, const char *lookfor);
LIBENTRY char  *strtok_r(register char *s, register const char *delim, char **lasts);
LIBENTRY char  *strupr(char *a);
LIBENTRY u32	strxfrm(char *s1, const char *s2, unsigned long n);

LIBENTRY int	vsprintf(char *buf, const char *fmt, va_list args);
LIBENTRY int	sprintf(char *str, const char *format, ...);
LIBENTRY int	printk(char *mask, ...);
LIBENTRY void	putc(char c);
LIBENTRY char	getc(void);
LIBENTRY u32	keysleft(void);
LIBENTRY u32	wait(u32 msecs, u32 poll_keysleft);
LIBENTRY u32	wait_ns(u32 nsecs);
LIBENTRY void	exit(u32 softreset);
#endif	// __USE_GLIBC__

#endif	//__ASSEMBLY__

#endif	// __LIBBIOS_H
