/***************************************************************************
    libBIOS - A portable library for developing an OpenSource BIOS
    Copyright (C) 2001 - David Aparicio

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Author can be reached at 'daparic@terra.es'
****************************************************************************/
// 
#include "dc21285.h"

#ifdef __ASSEMBLER__
	.macro	printz, reg
	LDR	r9, =\reg
9:	LDRB	r0, [r9]
	CMP	r0, #0
	BEQ	9f
	ADD	r9, r9, #1
	printCHR r0
	B	9b
9:	NOP
	.endm

	.macro	printCHR, reg
	MOV	r1, #ARMCSR_BASE
1:	LDR     r2, [r1,#OFS_UARTFLG]	/* check status*/
	TST     r2, #FLG_UART_TFIFOFULL
        BNE     1b
	STR	\reg, [r1,#OFS_UARTDR]
	LDR     \reg, [r1,#OFS_UARTFLG]
	.endm

	.macro	printCHRr	const
	MOV	r0, #\const
	printCHR r0
	.endm

	//	r0-r3, r8-r10
	.macro	printHEX
	MOV	r8, #0
2:
	MOV	r0, r3, LSR #28
	MOV	r3, r3, LSL #4
	CMP	r0, #9
	ADDGT	r0, r0, #('A'-10)	// mayor que 9
	ADDLE	r0, r0, #'0'		// menor igual que 9
	printCHR r0
	ADD	r8, r8, #1
	CMP	r8, #8
	BLT	2b
	.endm

	.macro	printHEXnl, register
	MOV	r3, \register
	printHEX
	MOV	r0, #'\r'
	printCHR r0
	MOV	r0, #'\n'
	printCHR r0
	.endm

	.macro	printHEXbl, register
	MOV	r3, \register
	printHEX
	MOV	r0, #' '
	printCHR r0
	.endm


	.macro	pushUSP
	STMDB	sp!, { sp }^
	NOP
	.endm

	.macro	popUSP
	LDMIA	sp!, { sp }^
	NOP
	.endm

	.macro	pushULR
	STMDB	sp!, { lr }^
	NOP
	.endm

	.macro	popULR
	LDMIA	sp!, { lr }^
	NOP
	.endm

	.macro	push, reglist
	STMDB	sp!, { \reglist }
	.endm

	.macro	pop, reglist
	LDMIA	sp!, { \reglist }
	.endm

	.macro	POINT, reg, mode
	LDR	\reg, =sys
	LDR	\reg, [\reg, #\mode]
	.endm

	.macro	saveTCBregs
	STMIA  	lr, { r0-r14 }^
	NOP		// No se puede acceder en este ciclo a SP-banked
	MRS	r0, spsr
	STR	r0, [lr, #O_FL]
	.endm

	.macro	loadTCBregs
	LDR	r0, [lr, #O_FL]
	MSR	spsr_c, r0			// CARGA flags y retorna
	LDMIA  	lr, { r0-r14 }^		// CARGA registros
	NOP		// No se puede acceder en este ciclo a SP-banked
	.endm

	.MACRO	_movi, reg, const32
	MOV	\reg,       #( \const32 &0xFF000000)
	ORR	\reg, \reg, #( \const32 &0x00FF0000)
	ORR	\reg, \reg, #( \const32 &0x0000FF00)
	ORR	\reg, \reg, #( \const32 &0x000000FF) 
	.ENDM

	.MACRO	_putc, char
	MOV	r0,#\char
	printCHR	r0
//	BL      putc
	.ENDM

	.MACRO	_putch, char
	_putc	\char
	_putc	'\r'
	.ENDM

	.MACRO	_keysleft, char
        MOV     r3, #ARMCSR_BASE
	LDR	r2, [r3, #OFS_UARTFLG]
	ANDS	r2, r2, #0x10
	MOVEQ	\char, #1		// if(keysleft) RETVALUE==1
	MOVNE	\char, #0
	.ENDM

	.MACRO	_getc, char
        MOV     r3, #ARMCSR_BASE
1:	LDR	r2, [r3, #OFS_UARTFLG]
	ANDS	r2, r2, #0x10
	BNE	1b
        LDR     \char, [r3, #OFS_UARTDR]
        LDR     r2, [r3, #OFS_RXSTAT]
        .ENDM
#endif
