
/***************************************************************************
    libBIOS - A portable library for developing an OpenSource BIOS
    Copyright (C) 2001 - David Aparicio

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Author can be reached at 'daparic@terra.es'
****************************************************************************/
// 
#include "emOS.h"

#undef DEPURACION

INLINE void _insTcbQ(tcb_t **tcbQ, tcb_t *tcb)
{
	tcb_t *aux = *tcbQ;

	if(aux==NULL)
	{
		*tcbQ  = tcb;
		tcb->n = tcb;
		tcb->p = tcb;
	}
	else
	{
		tcb->p = aux->p;
		tcb->n = aux;
		aux->p->n = tcb;
		aux->p = tcb;
	}
}

INLINE void _delTcbQ(tcb_t **tcbQ, tcb_t *tcb)
{
	tcb_t *aux;

	if(*tcbQ == tcb)
	{
		if(tcb == tcb->n)
		{
			*tcbQ = NULL;
			return;
		}
		else
			*tcbQ = tcb->n;
	}

	// Borrar de cola de libres
	aux       = tcb->n;
	aux->p    = tcb->p;
	tcb->p->n = aux;
}

INLINE void _tcbWakeup(tcb_t *tcb)
{	// Averiguar si este es mejor candidato que el actual
	if(	!sys.running ||
		tasks[tcb - sys.TCB].prio > tasks[sys.running - sys.TCB].prio)
			sys.running = tcb;
}

INLINE void _tcbSleep(tcb_t *tcb)
{	// Averiguar siguiente mejor candidato si este es el proceso en CPU
	if(tcb == sys.running)
	{
		register u32 prio = tasks[tcb - sys.TCB].prio;
		register int i;

		for(i=prio+1; i; i--)
			if(sys.ready[i-1])
			{
				sys.running = sys.ready[i-1];
				break;
			}
	}
}

#ifdef DEPURACION
const char *_tlab[] = {
	"Free", "Ready", "WaitS", "WaitR" };
#endif

void  tcbReady(tcb_t *tcb)
{
#ifdef DEPURACION
	register u32 old = tcb->status;
#endif
	if(!tcb) return;
	switch(tcb->status)
	{
	case STT_WAITS:
		break;
	case STT_WAITR:
//		_delTcbT(&sys.Ttimer, tcb);
		break;
	case STT_FREE:	// Especial: creacion de proceso...
		// Borrar de cola de libres
		_delTcbQ(&sys.Tfree, tcb);
		// Initialize Queues
		tcb->comm.get = 0;
		tcb->comm.put = 0;
		tcb->send.get = 0;
		tcb->send.put = 0;
		tcb->wait.get = 0;
		tcb->wait.put = 0;
		tcb->msg      = NULL;
		// Initialize timer references
		tcb->tmAsyn   = NULL;
		tcb->tmSync   = NULL;
		break;
	case STT_READY:	// Oops, we are trying to awake a non-sleeping
		printk("Ready(%i) -> Ready?? - panic\n", tcb-sys.TCB);
		while(1) {}
	default:
		return;
	}
	tcb->status = STT_READY;
	// Insertar en cola de ready
	_insTcbQ(&sys.ready[tasks[tcb-sys.TCB].prio], tcb);
	_tcbWakeup(tcb);
#ifdef DEPURACION
	printk("   %i.tcb(%i).Ready [was %s PC=%08X]\n", sys.tic, tcb-sys.TCB,
		_tlab[old&7], tcb->regs[I_PC]);
#endif
}

void  tcbWait(tcb_t *tcb, u8 **msg)
{
	register u32 old = tcb->status;

	tcb->status = msg ? STT_WAITR : STT_WAITS;
	tcb->msg    = msg;
	if(!tcb || old!=STT_READY)
	{
		printk("tcbWait%c(%i): Was task not ready? - panic\n",
			msg ? 'R' : 'S', tcb - sys.TCB);
		while(1) {}
	}

	_delTcbQ(&sys.ready[tasks[tcb-sys.TCB].prio], tcb);
	_tcbSleep(tcb);
#ifdef DEPURACION
	printk("   %i.tcb(%i).Wait%c [was %s PC=%08X]\n",sys.tic, tcb-sys.TCB,
		msg ? 'R' : 'S', _tlab[old&7], tcb->regs[I_PC]);
#endif
}

