
/***************************************************************************
    libBIOS - A portable library for developing an OpenSource BIOS
    Copyright (C) 2001 - David Aparicio

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Author can be reached at 'daparic@terra.es'
****************************************************************************/
// 
#include "emOS.h"

#undef DEPURACION
#undef DEPURACION_S

INLINE void _insTimer(timer_t **timerQ, timer_t *timer)
{
	register timer_t *ant, *aux;

	if(!*timerQ)
	{
		*timerQ = timer;
		timer->n = NULL;
		timer->p = NULL;
		return;
	}
	for(ant = NULL, aux = *timerQ; aux; ant = aux, aux = aux->n)
	{
		if(timer->tics_left < aux->tics_left)
		{		// Insertar delante de aux (medio o principio)
			if(!ant)	// Principio
			{
				aux->p    = timer;
				timer->n  = aux;
				timer->p  = NULL;
				*timerQ   = timer;
				aux->tics_left -= timer->tics_left;
			}
			else		// En medio
			{
				timer->n  = aux;
				timer->p  = ant;
				aux->p    = timer;
				ant->n    = timer;
				aux->tics_left -= timer->tics_left;
			}
			return;
		}
		else
			timer->tics_left -= aux->tics_left;
	}
	// Insertar al final
	ant->n   = timer;
	timer->p = ant;
	timer->n = NULL;
}

INLINE void _delTimer(timer_t **timerQ, timer_t *timer)
{
	register timer_t *aux;

	for(aux = *timerQ; aux; aux = aux->n)
	{
		if(aux==timer)
		{
			if(!aux->p)		// Borrar el primero
			{
				*timerQ = aux->n;
				if(aux->n)
				{
					aux->n->p = NULL;
					aux->n->tics_left += aux->tics_left;
				}
			}
			else if(!aux->n)	// Borrar el ultimo
			{
				aux->p->n = NULL;
			}
			else			// Borrar en medio
			{
				aux->p->n = aux->n;
				aux->n->p = aux->p;
				aux->n->tics_left += aux->tics_left;
			}
			return;
		}
	}
}

INLINE void _insTimerFree(timer_t **timerQ, timer_t *timer)
{
	timer_t *aux = *timerQ;

	if(aux==NULL)
	{
		*timerQ  = timer;
		timer->n = timer;
		timer->p = timer;
	}
	else
	{
		timer->p = aux->p;
		timer->n = aux;
		aux->p->n = timer;
		aux->p = timer;
	}
}

INLINE void _delTimerFree(timer_t **timerQ, timer_t *timer)
{
	timer_t *aux;

	if(*timerQ == timer)
	{
		if(timer == timer->n)
		{
			*timerQ = NULL;
			return;
		}
		else
			*timerQ = timer->n;
	}

	// Borrar de cola de libres
	aux       = timer->n;
	aux->p    = timer->p;
	timer->p->n = aux;
}

void   timerInsert(u32 tipo, void *xcb, u32 msecs)
{
	timer_t	*tm = sys.Efree;

	if(msecs==TM_NOWAIT || msecs==TM_NOTIMEOUT) return;
#ifdef DEPURACION
	switch(tipo)
	{
	case TIMER_MCB:
		printk("   %i.tcb(%i).mcb(%i).timer(%i) %i ms\n",
			sys.tic, sys.calling-sys.TCB,
			((mcb_t *)xcb)-sys.MCB, tm - sys.TMR, msecs);
		break;
	case TIMER_SYNC:
#ifdef DEPURACION_S
		printk("   %i.tcb(%i).sync.timer(%i) %i ms\n",
			sys.tic, ((tcb_t *)xcb)-sys.TCB, tm - sys.TMR, msecs);
#endif
		break;
	case TIMER_ASYN:
		printk("   %i.tcb(%i).asyn.timer(%i) %i ms\n",
			sys.tic, ((tcb_t *)xcb)-sys.TCB, tm - sys.TMR, msecs);
		break;
	default: break;
	}
#endif
	_delTimerFree(&sys.Efree, tm);
	switch(tipo)
	{
	case TIMER_SYNC:
		tm->i.tcb = xcb;
		((tcb_t *)xcb)->tmSync = tm;
		break;
	case TIMER_ASYN:
		tm->i.tcb = xcb;
		((tcb_t *)xcb)->tmAsyn = tm;
		break;
	case TIMER_MCB :
		tm->i.mcb = xcb;
		MCBHDR((mcb_t *)xcb).tmMcb = tm;
		break;
	default: return;	// Temporizador desconocido, salir
	}
	tm->tics_left = MSEC2TIC(msecs);
	tm->tipo      = tipo;
	_insTimer(&sys.timer, tm);
}

void   timerDelete(timer_t *tm)
{
	if(!tm) return;
	_delTimer(&sys.timer, tm);
	switch(tm->tipo)
	{
	case TIMER_SYNC:
#ifdef DEPURACION_S
		printk("   %i.tcb(%i).sync.timer(%i) done\n",
			sys.tic, tm->i.tcb - sys.TCB, tm - sys.TMR);
#endif
		tm->i.tcb->tmSync = NULL;
		break;
	case TIMER_ASYN:
#ifdef DEPURACION
		printk("   %i.tcb(%i).asyn.timer(%i) done\n",
			sys.tic, tm->i.tcb - sys.TCB, tm - sys.TMR);
#endif
		tm->i.tcb->tmAsyn = NULL;
		break;
	case TIMER_MCB:
#ifdef DEPURACION
		printk("   %i.tcb(%i).mcb(%i).timer(%i) done\n",
			sys.tic, sys.calling-sys.TCB,
			tm->i.mcb - sys.MCB, tm - sys.TMR);
#endif
		MCBHDR(tm->i.mcb).tmMcb  = NULL;
		break;
	default:
		break;
	}
	_insTimerFree(&sys.Efree, tm);
}

