
/***************************************************************************
    libBIOS - A portable library for developing an OpenSource BIOS
    Copyright (C) 2001 - David Aparicio

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Author can be reached at 'daparic@terra.es'
****************************************************************************/
// 
/* crt0.S
        Thanks to original codes in which this was inspired:
	 Digital uHAL   - Intel (c): Utilities information
	 Compaq bootldr - Compaq (c): Inicialization information
	 netwinder bios - Rebel.com (c): RAM detection & inicialization
	 ebsa285bios    - Russell King (GPL): RAM detection & inicialization
*/

#include "libBIOS.h"
#include "macros.h"
	
_start: .globl	_start
//--------------- Boot header for a cold reset (SA110)
	b		1f
	.long		SFS_ARCH_SA110
	.long		SFS_TYPE_POST
	.long		0		// entry
	.space		32		// label
	.long		_start		// baseC
	.long		0		// sizeC
	.long		0		// baseD
	.long		0		// sizeD

1:	ORR	pc, pc, #DC21285_FLASH	// make sure we're in Flash space
	MOV     r0, #SZ_1M
	STR	r0, [r0,#-1]		// deshabilita mapeo RAM/ROM

	/*; Turn off all interrupts*/
	MOV	ip, #ARMCSR_BASE
	MOV     r0, #0xFFFFFFFF	/* into r0 to clear all interrupt enables*/
	STR	r0, [ip, #OFS_IRQ_DISABLE]
	STR	r0, [ip, #OFS_FIQ_DISABLE]

	/*; Initialize UART (38400N8) */
	MOV	r0, #0		// disable UART completely, no HP SIR or IrDA
	STR	r0, [ip,#OFS_UARTCON]
	MOV	r0, #0x13	// 38400 we don't check clock settings (50 MHz) 
	STR	r0, [ip,#OFS_L_UBRLCR]
	MOV	r0, #0x00
	STR	r0, [ip,#OFS_M_UBRLCR]
	MOV	r0, #0x70
	STR	r0, [ip,#OFS_H_UBRLCR]
	MOV	r0, #1		/*; set enable bit in UARTCON register*/
	STR	r0, [ip,#OFS_UARTCON]

	MOV	r0, #0		// Reset PCI
	STR	r0, [ip,#OFS_SA110_CNTL]

	BL	sdram_init
	mov	r3, #ARMCSR_BASE
	str	r0, [r3, #OFS_MBOX1]	// Save "ramsize"

/*
 * Now setup caches
 */
	mrc	p15, 0, r0, c1, c0	@ read control reg
	bic	r0, r0, #0x0d		@ D-cache, wback & MMU off
	orr	r0, r0, #1 << 12	@ I-cache on
	mcr	p15, 0, r0, c1, c0	@ Control reg

	bl	beginSingle

	mov	r3, #ARMCSR_BASE
	ldr	r1, [r3, #OFS_MBOX1]	// Load "ramsize"
	sub	r0, r1, #128
	str	r0, [r3, #OFS_MBOX0]	// (env_t *) at (ramsize - 128B)
	str	r1, [r0]		// Save ramsize in "env" structure
	mov	r2, #0
	str	r2, [r3, #OFS_MBOX2]
	str	r2, [r3, #OFS_MBOX3]
	bl	entry

exit:	.globl	exit			// arg==0 then JUMP FLASH
	bl	haltSingle		// arg!=0 then JUMP LOADER
2:	b	2b

