
/***************************************************************************
    libBIOS - A portable library for developing an OpenSource BIOS
    Copyright (C) 2001 - David Aparicio

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Author can be reached at 'daparic@terra.es'
****************************************************************************/
// 
#include "libBIOS.h"
#include "dc21285.h"

u32 pci_bus2ram(u32 bus_addr)
{
	return (bus_addr | DC21285_PCI_MEM);
}

u32 pci_bus2io(u16 bus_addr)
{
	return (bus_addr | DC21285_PCI_IO);
}

u32 pci_ram2bus(u32 ram_addr)
{
	if(ram_addr > *CSR_MBOX1)
		return 0xFFFFFFFF;
	else
		return (*CSR_PCISDRAMBASE&0xFFFFFFF0)+ram_addr;
}

u8   pci_rm8 (u32 port) { return *PCIMEM  (port); }
u16  pci_rm16(u32 port) { return *PCIMEM16(port); }
u32  pci_rm32(u32 port) { return *PCIMEM32(port); }

void pci_wm8 (u32 port,  u8 val) { *PCIMEM  (port) = val; }
void pci_wm16(u32 port, u16 val) { *PCIMEM16(port) = val; }
void pci_wm32(u32 port, u32 val) { *PCIMEM32(port) = val; }

u8   pci_rp8 (u32 port) { return *PCIPORT  (port); }
u16  pci_rp16(u32 port) { return *PCIPORT16(port); }
u32  pci_rp32(u32 port) { return *PCIPORT32(port); }

void pci_wp8 (u32 port,  u8 val) { *PCIPORT  (port) = val; }
void pci_wp16(u32 port, u16 val) { *PCIPORT16(port) = val; }
void pci_wp32(u32 port, u32 val) { *PCIPORT32(port) = val; }

int pci_cnvParCsr(u8 bus, u8 slot, u8 fun, csr_t **csr)
{
	if (bus == 0) {
		if(slot == 0 && fun == 0)
			*csr = (csr_t *)(DC21285_ARMCSR_BASE);
		else if (slot < 21 )
			*csr = (csr_t *)(DC21285_PCICFG0  | 0xc00000 |
				((slot & 0x1F) << 11) | ((fun & 0x07)  << 8));
		else
			*csr = CSRNUL;
	} else
		*csr = (csr_t *)(DC21285_PCICFG1  |
				((bus & 0xFF) << 16) |
				((slot& 0x1F) << 11) |
				((fun & 0x07) <<  8));
	return 0;
}

int pci_cnvCsrPar(csr_t *csr, u8 *bus, u8 *slot, u8 *fun)
{
	u32 base = (u32)csr;

	if((base&0xFF000000)==DC21285_PCICFG0)
	{
		*bus   = 0;
		*slot  = (base&0x0000FF00)>>8;
		*fun   = (*slot & 0x07);
		*slot  = *slot >> 3;
	}
	else if((base&0xFF000000)==DC21285_PCICFG1)
	{
		*bus   = (base&0x00FF0000)>>16;
		*slot  = (base&0x0000F800)>>11;
		*fun   = (base&0x00000700)>> 8;
	}
	else
	{
		*bus = *slot = *fun = 0;
	}
	return 0;
}

int pci_getNumSlot(csr_t *csr)
{
	u32 base = (u32)csr;

	if((base&0xFF000000)==DC21285_PCICFG0)
		return (base & 0x000FFFFF) >> 11;
	else if((base&0xFF000000)==DC21285_PCICFG1)
		return (base & 0x0000FF00) >> 8;
	else
		return 0;
}

