
/***************************************************************************
    libBIOS - A portable library for developing an OpenSource BIOS
    Copyright (C) 2001 - David Aparicio

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Author can be reached at 'daparic@terra.es'
****************************************************************************/
// 
#include "macros.h"

sdram_init:	.globl	sdram_init
#define SDRAM_NRARRAYS	4
	mov	r8, lr	// Save RET address
	/*
	;; ********************************************************************
	;; sdram_init -- Inicializa refresco SDRAM, tamao SDRAM
        ;;   Destruye todos los registros, retorna en r0 el tamao de la RAM...
	;;********************************************************************
	*/

	mov	r0, #0
	mcr	p15, 0, r0, c7, c7, 0	@ flush all caches
/*
 * We need all caches off for the SDRAM initialisation -
 * we have s/w timing loops to time the SDRAM refresh
 * cycles in here, which must be synchronised to the bus
 * clock.
 */
	mrc	p15, 0, r0, c1, c0	@ Control reg
	bic	r0, r0, #0x0d		@ D-cache, wback & MMU off
	bic	r0, r0, #1 << 12	@ I-cache off
	mcr	p15, 0, r0, c1, c0	@ Control reg

/*
 * Initialise SDRAM.  This copes with the SDRAM in any state -
 * it does a complete initialisation and allocation of all
 * banks.  We basically follow the sequence given in the
 * EBSA285 manual.
 *
 * Turn off SDRAM refresh
 */
	mov	r0, #0
	str	r0, [ip, #OFS_SDRAMTIMING]
/* 
 * Wait for any refresh cycles to complete
 */
	mov	r0, #64
1:	subs	r0, r0, #1
	bgt	1b

/*
 * All-banks precharge SDRAM arrays
 */
	mov	r0, #DC21285_SDRAM_CTRL
	mov	r1, #SDRAM_NRARRAYS
1:	ldr	r2, [r0]
	ldr	r2, [r0]
	add	r0, r0, #0x4000		@ next ARRAY
	subs	r1, r1, #1
	bgt	1b
/*
 * Write mode registers
 */
	mov	r0, #DC21285_SDRAM_CTRL
	orr	r0, r0, #0x88		@ Tcas
	mov	r1, #SDRAM_NRARRAYS
1:	str	r0, [r0]
	add	r0, r0, #0x4000
	subs	r1, r1, #1
	bgt	1b
/*
 * Turn on minimum SDRAM refresh
 */
	mov	r0, #0x00010000
	orr	r0, r0, #0x00000b00
	orr	r0, r0, #0x000000a5
	str	r0, [ip, #OFS_SDRAMTIMING]

/*
 * Set size to 64MB, space banks at 64MB intervals, with multiplexer mode 4
 */
	mov	r1, #SDRAM_NRARRAYS
	add	r2, ip, #OFS_SDRAMADDRSIZE0
	mov	r0, #0x4f
1:	str	r0, [r2], #4
	add	r0, r0, #64*1024*1024
	subs	r1, r1, #1
	bgt	1b
/*
 * Wait for banks to be refreshed
 */
	mov	r0, #10
	mov	r1, #0
1:	subs	r1, r1, #2
	bgt	1b
	subs	r0, r0, #1
	bgt	1b

/*
 * Set SDRAM refresh to normal
 */
	mov	r0, #0x001a0000
	orr	r0, r0, #0x00000b00
	orr	r0, r0, #0x000000a5
	str	r0, [ip, #OFS_SDRAMTIMING]

	mov	r4, #0x00000000
1:	mov	r5, #0
/*
 * Detect RAM multiplexer settings.
 *
 * Theory of operation:
 *  There are 6 distinct multiplexer settings.  It is possible to detect
 *  which multiplexer settings are correct for the SDRAM array by testing
 *  just four bits with the multiplexer set to mode 4:
 *   row address 9
 *   row address 10
 *   row address 11
 *   bank address 1
 *  we test to see if toggling these address lines are used by the SDRAM,
 *  and select the multiplexer setting from the result.
 *
 * Multiplexer 	BA1	RA11	RA10	RA9	code	Max array size
 *  000 1 (08)	-	-	-	-	0	1MB
 *  000 0 (00)	-	-	-	used	1	2MB
 *  001 x (10)	-	-	used	used	3	16MB
 *  010 x (20)	-	used	used	used	7	64MB
 *  011 x (30)	used	-	used	used	b	8MB	
 *  100 x (40)	used	used	used	used	f	64MB
 */

	mov	r0, r4			@ check for presence
	add	r1, r4, #64
	bl	_tstAlias
	bne	next_bank		@ array not present - disable

	mov	r0, r4			@ check bit 20 (BA1)
	orr	r1, r0, #1 << 20
	bl	_tstAlias
	orreq	r5, r5, #8		@ bit 20 (BA1) has an effect

	mov	r0, r4			@ check bit 22 (row 11)
	orr	r1, r0, #1 << 22
	bl	_tstAlias
	orreq	r5, r5, #4		@ bit 22 (row 11) has an effect

	mov	r0, r4			@ check bit 21 (row 10)
	orr	r1, r0, #1 << 21
	bl	_tstAlias
	orreq	r5, r5, #2		@ bit 21 (row 10) has an effect

	mov	r0, r4			@ check bit 18 (row 9)
	orr	r1, r0, #1 << 18
	bl	_tstAlias
	orreq	r5, r5, #1		@ bit 18 (row 9) has an effect

	adr	r1, ram_modes		@ convert test result to mux setting
	ldrb	r6, [r1, r5]
	teq	r6, #0xff		@ was the test result illegal?
	moveq	r5, #0			@ if it was, disable the array
	beq	next_bank

	orr	r6, r6, r4		@ incorporate address
	add	r0, ip, #OFS_SDRAMADDRSIZE0	@ set mux correctly
	orr	r5, r6, #7		@ leave array size at 64MB
	str	r5, [r0, r4, lsr #24]
/*
 * Detect RAM array size
 */
	mov	r5, #1 << 20		@ start at 1MB
	mov	r7, #1

2:	mov	r0, r4			@ check bit (r7 + 19)
	add	r1, r4, r5
	bl	_tstAlias
	bne	3f			@ if bit (r7 + 19) doesnt work
	mov	r5, r5, lsl #1
	cmp	r7, #7
	addne	r7, r7, #1
	bne	2b

3:	orr	r5, r6, r7

next_bank:
	add	r0, ip, #OFS_SDRAMADDRSIZE0
	str	r5, [r0, r4, lsr #24]	@ set array size

	add	r4, r4, #0x04000000	@ next bank address
	teq	r4, #0x10000000
	bne	1b

/*
 * Now that we know the size of each SDRAM array, allocate
 * their addresses, starting at the largest size and working
 * progressively smaller.
 *  r4 is our "top of memory"
 */
	mov	r3, #7			@ start at largest size
	mov	r4, #0x00000000
1:	mov	r5, #SDRAM_NRARRAYS
	add	r2, ip, #OFS_SDRAMADDRSIZE0
2:	ldr	r0, [r2], #4		@ Read SDRAM size & addr
	and	r1, r0, #7
	teq	r1, r3
	bne	3f

	and	r0, r0, #127		@ Preserve size etc
	orr	r0, r0, r4		@ Add base address
	str	r0, [r2, #-4]		@ Write SDRAM size & addr

	mov	r0, #524288
	add	r4, r4, r0, lsl r1	@ Next address

3:	subs	r5, r5, #1		@ Next bank
	bgt	2b
	subs	r3, r3, #1		@ Next size
	bgt	1b

	// ramsize initialize
	MOV	r0, r4			// ramsize in R0
	MOV	pc, r8			// Return from saved address

	/*
	;; *******************************************************************
 	;; _tstAlias -- Test to see if two RAM locations are aliases.
 	;;  r0 = location 1
 	;;  r1 = location 2
 	;;  destroys { r10, r11 }
	;;  returns {flags}
	;; *******************************************************************
 	*/
_tstAlias:
	mov	r10, #0x55
	orr	r10, r10, #0xaa00
	orr	r10, r10, r10, lsl #16	@ r10 = 0xaa55aa55
	str	r10, [r0]		@ store 0xaa55aa55 to [r0]
	mvn	r10, r10
	str	r10, [r1]		@ store 0x55aa55aa to [r1]
	mvn	r10, r10
	ldr	r11, [r0]
	teq	r10, r11		@ is [r0] 0xaa55aa55?
	bne	1f			@ no - this means r1 is an alias of r0
	mvn	r10, r10
	str	r10, [r0]		@ store 0x55aa55aa to [r0]
	mvn	r10, r10
	str	r10, [r1]		@ store 0xaa55aa55 to [r1]
	mvn	r10, r10
	ldr	r11, [r0]
	teq	r10, r11		@ is [r0] 0x55aa55aa?
1:	mov	pc, lr

//---------------------------------------------------------------------------
ram_modes:				@ BA1  RA11 RA10 RA9
	.byte	0x08			@  -    -    -    -
	.byte	0x00			@  -    -    -    x
	.byte	0xff			@  -    -    x    - (illegal)
	.byte	0x10			@  -    -    x    x
	.byte	0xff			@  -    x    -    - (illegal)
	.byte	0xff			@  -    x    -    x (illegal)
	.byte	0xff			@  -    x    x    - (illegal)
	.byte	0x20			@  -    x    x    x
	.byte	0xff			@  x    -    -    - (illegal)
	.byte	0xff			@  x    -    -    x (illegal)
	.byte	0xff			@  x    -    x    - (illegal)
	.byte	0x30			@  x    -    x    x
	.byte	0xff			@  x    x    -    - (illegal)
	.byte	0xff			@  x    x    -    x (illegal)
	.byte	0xff			@  x    x    x    - (illegal)
	.byte	0x40			@  x    x    x    x
//---------------------------------------------------------------------------

