
/***************************************************************************
    libBIOS - A portable library for developing an OpenSource BIOS
    Copyright (C) 2001 - David Aparicio

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Author can be reached at 'daparic@terra.es'
****************************************************************************/
// 
#include "libBIOS.h"
#include "macros.h"

//-------------------------------------------------------------------------
//---- SingleUser routines. This mode is STANDALONE (SPV monotask)
//----     1) SP initialization for SPV only
//----     2) TIMER1 initialization

beginSingle:	.globl	beginSingle
	// Enable caches (CODE only)
	mov	r0, #0
	mcr	p15, 0, r0, c7, c7, 0	@ flush all caches
	mrc	p15, 0, r0, c1, c0	@ Control reg
	bic	r0, r0, #0x0d		@ D-cache, wback & MMU off
	orr	r0, r0, #1 << 12	@ I-cache on
	mcr	p15, 0, r0, c1, c0	@ Control reg

        MOV     r3, #ARMCSR_BASE

        // Disable all interrupts
        MOV     r0, #0xFFFFFFFF
        STR     r0, [r3, #OFS_IRQ_DISABLE]
        STR     r0, [r3, #OFS_FIQ_DISABLE]

	// Clean SERIAL input
	MOV	r10, lr			// save LR
1:	BL	keysleft
	BNE	2f
	BL	getc
	B	1b
2:
        MOV     r3, #ARMCSR_BASE

	//  Initialize SP (from end of memory minus env_t space)
        LDR     r0, [r3, #OFS_MBOX1]    // ramsize (arg1)
        MOV     fp, #0
        SUB     sp, r0, #0x1000		// leaves space for FIQ, IRQ and USR sp

	// Enable TIMER1 (millisecs counter)
	MOV	r0, #TIMER_CNTL_ENABLE|TIMER_CNTL_DIV16
	ORR	r0, r0, #TIMER_CNTL_AUTORELOAD
	STR	r0, [r3, #OFS_TIMER1_CNTL]

	// Enable TIMER2 (microsecs counter)
	MOV	r0, #TIMER_CNTL_ENABLE
	ORR	r0, r0, #TIMER_CNTL_AUTORELOAD
	STR	r0, [r3, #OFS_TIMER2_CNTL]

        // Prepare for high-level C call
	LDR	r0, [r3, #OFS_MBOX0]	// env_t   (arg0)
        MOV     pc, r10

haltSingle:	.globl	haltSingle
        MOV     r3, #ARMCSR_BASE
	// Disable all counters
	MOV	r2, #0
	STR	r2, [r3, #OFS_TIMER2_CNTL]
	STR	r2, [r3, #OFS_TIMER1_CNTL]

	//--- Address to JUMP in R0
//	CMP	r0, #0
//	LDREQ	lr, =addr_flash		// Jump to flash
//	LDRNE	lr, =addr_load		// Jump to download zone
//	LDR	lr, [lr]
	MOV	lr, r0
        MOV     pc, lr

addr_flash:	.long	DC21285_FLASH

//--- WATCHOUT!! All counters are related to a 50MHz clock (fclk_in)
#define UPPER_TIME	0x10000
tic_limit:	.long	UPPER_TIME - 2918
mtc_limit:	.long	UPPER_TIME - 50

//--- wait nanosec-resolution (usually for hardware delays - flash programming)
wait_ns:	.globl	wait_ns
        MOV     r3, #ARMCSR_BASE
	push	r4
	LDR	r4, =mtc_limit
	LDR	r4, [r4]
	//			if(!msecs) return 0;
1:	CMP	r0, #0
	BEQ	4f		// RETVALUE==0
	//			msecs--
	SUB	r0, r0, #1
	//			TIMER1_LOAD = UPPER_TIME
	MOV	r2, #UPPER_TIME
        STR     r2, [r3, #OFS_TIMER2_LOAD]
	//			while(time_elapsed_TIMER2 < 50)
3:	LDR	r2, [r3, #OFS_TIMER2_VALUE]
	CMP	r4, r2
	BLE	3b
	B	1b

4:	pop	r4
	MOV     pc, lr

//--- void wait(u32 msecs, u32 rs_in_interruptable)
//          msecs: Millisecs to wait
//          rs_in_int: Non-zero if serial incoming data should stop waiting
wait:	.globl	wait
        MOV     r3, #ARMCSR_BASE
	push	r4
	LDR	r4, =tic_limit
	LDR	r4, [r4]
	//			if(!msecs) return 0;
1:	CMP	r0, #0
	BEQ	4f		// RETVALUE==0
	//			msecs--
	SUB	r0, r0, #1
	//			TIMER1_LOAD = UPPER_TIME
	MOV	r2, #UPPER_TIME
        STR     r2, [r3, #OFS_TIMER1_LOAD]
	//			if(arg1 && thereare_keys) return 1;
2:	CMP	r1, #0
	BEQ	3f
	LDR	r2, [r3, #OFS_UARTFLG]
	ANDS	r2, r2, #0x10
	MOVEQ	r0, #1		// RETVALUE==1
	BEQ	4f
	//			while(time_elapsed_TIMER1 < 2918)
3:	LDR	r2, [r3, #OFS_TIMER1_VALUE]
	CMP	r4, r2
	BLE	2b
	B	1b

4:	pop	r4
	MOV     pc, lr

initTimer: .globl  initTimer     // En R0 los tics por interrupcion
        // --- Inicializar TIMER1 para TICK
        MOV     r1, #ARMCSR_BASE
        // en R0 los tics por interrupcion
        STR     r0, [r1, #OFS_TIMER1_LOAD]
        MOV     r0, #FLG_IRQ_TIMER1
        STR     r0, [r1, #OFS_IRQ_ENABLE]
        MOV     r0, #0
        STR     r0, [r1, #OFS_TIMER1_CLR]
        MOV     pc, lr

keysleft:	.globl	keysleft
	_keysleft	r0
	MOV	pc, lr

putc:	.globl  putc
	printCHR r0
        MOV     pc, lr

getc:	.globl  getc
	_getc	r0
        MOV     pc, lr

pci_isHost:	.globl	pci_isHost
	mov	r3, #ARMCSR_BASE
	ldr	r0, [r3, #OFS_SA110_CNTL]
	and	r0, r0, #0x80000000
	mov	pc, lr
