
/***************************************************************************
    libBIOS - A portable library for developing an OpenSource BIOS
    Copyright (C) 2001 - David Aparicio

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Author can be reached at 'daparic@terra.es'
****************************************************************************/
// 
#include "libBIOS.h"

typedef short Align;

//#define CC_DEPU

#ifdef CC_DEPU
#  define dbg_msj(x) printf x
#else
#  define dbg_msj(x)
#endif

typedef union header
   {
   struct
      {
      union header *ptr;
      unsigned      size;
      } s;
   Align x;
   } Header;

Header  *freep;
unsigned UniLibres;
unsigned UniOcupados;
unsigned MaxUniOcu;

LIBENTRY void IniHeap(unsigned frp, unsigned sizeheap)
{
freep=(Header *)frp;

freep->s.ptr=NULL;
freep->s.size= sizeheap/sizeof(Header);

dbg_msj(("Inicializado heap:(%08X,%lu bytes); %lu unidades de %u bytes\n",
        frp, sizeheap,
        sizeheap/sizeof(Header),
        sizeof(Header)));

UniLibres=sizeheap/sizeof(Header);
UniOcupados=MaxUniOcu=0;
}


LIBENTRY void * malloc( unsigned long nbytes )
{
Header    *p;
Header    *prevp;
unsigned   numunits;

numunits = (nbytes+sizeof(Header)-1)/sizeof(Header)+1;

UniLibres-=numunits;
UniOcupados+=numunits;

if (UniOcupados>MaxUniOcu) MaxUniOcu=UniOcupados;

dbg_msj(("Solicit: %-6li bytes %-6li unid L%-6li O%-6li Omax%-6li\n",
               nbytes,
               numunits,
               UniLibres,
               UniOcupados,
               MaxUniOcu));

for (prevp=NULL,p=freep;
     p!=NULL;
     prevp=p, p=p->s.ptr)
   {
   if (p->s.size >= numunits)
      {
      Header    *paux;

      if (p->s.size > numunits)
         {
         paux=p+numunits;
         paux->s.ptr=p->s.ptr;
         paux->s.size=p->s.size-numunits;
         }
      else paux=p->s.ptr;
      if (prevp==NULL) freep=paux;
      else prevp->s.ptr=paux;
      p->s.size=numunits;
      p->s.ptr=NULL;
      return (void *) (p+1);
      }
   }
return (void *) p;
}


LIBENTRY void free (void *ap)

{
Header    *bp;
Header    *p;
Header    *prevp;

bp = (Header *) ap -1;

UniLibres+=bp->s.size;
UniOcupados-=bp->s.size;

dbg_msj(("Devueltos: %lu unidades L%lu O%lu\n",
        bp->s.size,
        UniLibres,
        UniOcupados));

if (freep==NULL) freep=bp;
else
   {
   for (prevp=NULL, p=freep;
        p!=NULL;
        prevp=p, p=p->s.ptr)
      {
      if (bp < p)
         {
         if (prevp==NULL)
            {
            bp->s.ptr=freep;
            freep=bp;
            }
         else if (prevp+prevp->s.size==bp)
            {
            prevp->s.size+=bp->s.size;
            bp=prevp;
            }
         else
            {
            bp->s.ptr=prevp->s.ptr;
            prevp->s.ptr=bp;

            }
         if (bp+bp->s.size==p)
            {
            bp->s.size+=p->s.size;
            bp->s.ptr=p->s.ptr;
            }
         return;
         }
      }
   if (prevp+prevp->s.size==bp)
      {
      prevp->s.size+=bp->s.size;
      }
   else prevp->s.ptr=bp;
   }
}
