
/***************************************************************************
    libBIOS - A portable library for developing an OpenSource BIOS
    Copyright (C) 2001 - David Aparicio

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Author can be reached at 'daparic@terra.es'
****************************************************************************/
// 
#include "libBIOS.h"

#undef DEPURACION

LIBENTRY int pci_grpInit(csrGrp_t g)
{
	g[0].csr = CSRNUL;
	return 0;
}

LIBENTRY void pci_grpAdd(csrGrp_t g, csr_t *csr)
{
	u32 n[PCI_GRPMAX];
	u32 i, j;

#ifdef DEPURACION
	u8 bus, dev, fun;
	pci_cnvCsrPar(csr, &bus, &dev, &fun);
#endif
	if(csr->vendor==0x0000 || csr->vendor==0xFFFF ||
	   csr->device==0x0000 || csr->device==0xFFFF)
		return;
	for(i=0; g[i].csr; i++) {}
	if(i==MAXCSR) return;

	memset(    n, 0, sizeof(n));
	memset(g[i].addr, 0, sizeof(g[i].addr));
	g[i  ].csr   = csr;
	g[i+1].csr = CSRNUL;

	if(PCI_CSRHDR0(csr))
	{
		for(j=0; j<=6; j++)
		{
			register u32 t, *a, s;

 			a = j<6
				? &(csr->u.hdr0.baseaddr[j])
				: &(csr->u.hdr0.rom_base);
			t = pci_getType(a);
			if(t==0xFFFFFFFF)	// 64 bits. We don't handle
			{
				j++;		// skip half 32-bit address
				continue;
			}
			s = pci_getSize(a);
			if(s)
			{
#ifdef DEPURACION
printk("0.[%i,%2i].bar[%i] type %i size %i\n", bus, dev, j, t, s);
#endif
				g[i].addr[t][n[t]] = a;
				g[i].size[t][n[t]] = s;
				n[t]++;
			}
		}
#ifdef DEPURACION
printk("0.[%i,%2i] %i %i %i %i %i\n",
	bus, dev, n[0], n[1], n[2], n[3], n[4]);
#endif
	}
	if(PCI_CSRHDR1(csr))
	{
		for(j=0; j<=2; j++)
		{
			register u32 t, *a, s;

 			a = j<2
				? &(csr->u.hdr1.baseaddr[j])
				: &(csr->u.hdr1.rom_base);
			t = pci_getType(a);
			s = pci_getSize(a);
			if(s)
			{
#ifdef DEPURACION
printk("1.[%i,%2i].bar[%i] type %i size %i\n", bus, dev, j, t, s);
#endif
				g[i].addr[t][n[t]] = a;
				g[i].size[t][n[t]] = s;
				n[t]++;
			}
		}
#ifdef DEPURACION
printk("1.[%i,%2i] %i %i %i %i %i\n",
	bus, dev, n[0], n[1], n[2], n[3], n[4]);
#endif
	}
#ifdef DEPURACION
	printk("-> ");
	for(j=0; j<PCI_GRPMAX; j++)
	{
		u32 k, x;
		for(x=0, k=0; g[i].addr[j][k]; k++) x++;
		printk("%i(%i) ", x, n[j]);
	}
	printk("\n");
#endif
}

LIBENTRY u32 pci_getType(volatile u32 *addr)
{
	//--- We group ROM and MEM as same non-prefetchable, writable
	//--- memory. We do so because PCI/PCI bridges can not distinguish
	//--- them, and address mask resolution is easier this way.
	//--- There is a restriction with low 1Mbyte ISA, that we don't
	//--- guarantee proper assignment outside main BUS0 (again because
	//--- of mask resolution). Also a lot of problems for hot-plugging...
	//--- Oops. We hope somebody smarter than us fix it in a proper way.
	//--- I would like to hear from this
	//--- daparic@terra.es	-	26/01/01

	if(*addr==0xFFFFFFFF)
		return *addr;	// VOID

	if(((u32)addr&0xFF) > 16+24)	// Points to past baseaddr zone in CSR
		return PCI_GRPROM;	// ROM
	if(*addr&1)
		return PCI_GRPIO;	// I/O
	if((*addr&6) == 4)		// 64 bit resource. we don't support
		return 0xFFFFFFFF;
	if((*addr&6) == 2)
		return PCI_GRPISA;	// ISA
	if(*addr&8)
		return PCI_GRPRAM;	// prefetchable!! -> RAM

	return PCI_GRPMEM; 		// MEM
}

LIBENTRY char *pci_strType(volatile u32 *addr)
{
	if(*addr==0xFFFFFFFF)
		return NULL;	// VOID

	if(((u32)addr&0xFF) > 16+24)	// Points to past baseaddr zone in CSR
		return "ROM";	// ROM: Non prefetchable, writable (FLASH)
	if(*addr&1)
		return "I/O";	// I/O: Usual PCI I/O
	if((*addr&6) == 4)		// 64 bit resource. we don't support
		return "M64";
	if((*addr&6) == 2)
		return "ISA";	// ISA: We use as non-prefetch, BRIDGES watchout
	if(*addr&8)
		return "RAM";	// RAM: Prefetchable

	return "MEM";		// MEM: Non prefetchable, usually mapped I/O
}

LIBENTRY u32 pci_getAddr(volatile u32 *addr)
{
	if(((u32)addr&0xFF) > 16+24)	// past baseaddr zone in CSR
		return (*addr & 0xFFFFF800);	// ROMADDR
	else if(*addr & 1)	// I/O
		return (*addr & 0x0000FFFC);
	else
		return (*addr & 0xFFFFFFF0); 
}

LIBENTRY u32 pci_getSize(volatile u32 *addr)
{
	u32 tmp, tmp2;
	
	tmp  = *addr;
	*addr= 0xFFFFFFFF;
	tmp2 = *addr;
	*addr= tmp;
	if((tmp2!=0xFFFFFFFF) && (tmp!=tmp2))
	{
		if(((u32)addr&0xFF) > 16+24)	// past baseaddr zone in CSR
			tmp = (~(tmp2 & 0xFFFFF800)) + 1;	// ROMADDR
		else if(tmp2 & 1)	// I/O
			tmp = (-(tmp2 & 0xFFFFFFFC)) & 0xFFFF;
		else
			tmp = (~(tmp2 & 0xFFFFFFF0)) + 1; 
	}
	else if(tmp&0xFFFFFFF0)	// Incompatible PCI: ALI IDE 5229. Sig.
		tmp = 8;
	else			// Zero value.
		tmp = 0;
	return tmp;
}

LIBENTRY void pci_grpScan(csrGrp_t grp)
{
	u16 bus, slot, func;
	int cards;

	cards = 0;
	pci_grpInit(grp);
	for(bus=0; bus<3; bus++)
	  for(slot=0; slot<32; slot++) for(func=0; func<8; func++)
	{
		csr_t *csr;

		csr = NULL;
		pci_cnvParCsr(bus, slot, func, &csr);

		if(!csr || (csr->vendor==0xFFFF && csr->device==0xFFFF))
			continue;
		// Si no es multifuncion, mirar solo func: 0
		if((csr->header & 0x80) == 0 && func==1)
			break;
		pci_grpAdd(grp, csr);
		cards++;
	}
}

LIBENTRY pcigrp_t *pci_grpGetDev (csrGrp_t g, int c)
{
	return g+c;
}

LIBENTRY u32      *pci_grpGetAddr(csrGrp_t g, int c, int t, int r)
{
	return g[c].addr[t][r];
}

LIBENTRY u32       pci_grpGetSize(csrGrp_t g, int c, int t, int r)
{
	return g[c].size[t][r];
}

LIBENTRY csr_t *pci_grpFindIndex(csrGrp_t csrg, int i)
{
	return csrg[i].csr;
}

LIBENTRY csr_t * pci_grpFindClass(csrGrp_t grp, u16 classpair)
{
	register unsigned i;

	for(i=0; grp[i].csr; i++)
	{
		if(PCI_CLASS(grp[i].csr) == classpair)
			return grp[i].csr;
	}
	return CSRNUL;
}

LIBENTRY csr_t * pci_grpFindDevice(csrGrp_t g, u16 vendor, u16 device)
{
	register unsigned i;

	for(i=0; g[i].csr; i++)
	{
		if(g[i].csr->vendor==vendor && g[i].csr->device==device)
			return g[i].csr;
	}
	return CSRNUL;
}
