
/***************************************************************************
    libBIOS - A portable library for developing an OpenSource BIOS
    Copyright (C) 2001 - David Aparicio

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Author can be reached at 'daparic@terra.es'
****************************************************************************/
// 
#include "libBIOS.h"

#undef  DEPURACION
#undef DEPURACION_E

//--- Global "discovered buses counter"
static int    nbus;

//--------------------------------------------------------------------------
static void discover_bus(csrGrp_t grp, int bus,
			int prv[PCI_BUSMAX], csr_t *act[PCI_BUSMAX])
{
        int dev, tmp;

        for(dev=0; dev < PCI_DEVMAX; dev++)
        {
                csr_t *csr;

                pci_cnvParCsr(bus, dev, 0, &csr);
                if( !csr || !PCI_CSRHDR1(csr) )
                        continue;
#ifdef DEPURACION
        printk("Found bridge(%i,%i) at device(%i)\n", bus, nbus, dev);
#endif
		// Enable bridge (at least for configuration cycles...)
                csr->u.hdr1.busE   = bus;
                csr->u.hdr1.busI   = nbus;
                csr->u.hdr1.busmax = nbus;

                prv[nbus] = bus;
                act[nbus] = csr;
                nbus++;
		pci_grpAdd(grp, csr);
		pci_devScan(csr);	// bridge device
                for(tmp=bus; tmp; tmp = prv[tmp]) act[tmp]->u.hdr1.busmax++;
                discover_bus(grp, nbus-1, prv, act);
        }
}

static int pci_init(pciFrame_t *mf)
{
        u8 bus, dev, fun;
	u32  i, j, k;
        int nadd;
        csrGrp_t grp;
        csr_t *csr;
	int    pbus[PCI_BUSMAX];
	csr_t *abus[PCI_BUSMAX];
	// Temporary assignment structures...
	u32  n[PCI_BUSMAX][PCI_GRPMAX];		// Number of resources type X
	u32  d[PCI_BUSMAX][PCI_GRPMAX][MAXCSR];	// DeviceIndex of res.
	u32 *p[PCI_BUSMAX][PCI_GRPMAX][MAXCSR];	// Address of res (csr pointer).
	u32  a[PCI_BUSMAX][PCI_GRPMAX][MAXCSR];	// Address of res (value).
	u32  s[PCI_BUSMAX][PCI_GRPMAX][MAXCSR];	// Size    of res.
	u32  fb[PCI_BUSMAX][PCI_GRPMAX];	// Bridge FrameBase
	u32  fl[PCI_BUSMAX][PCI_GRPMAX];	// Bridge FrameLimit

        //--- Initialize and discover (BUSES)
	//--- arbitrer for busX is stored at index X
        pci_grpInit(grp);
	nadd	= 0;
        nbus    = 1;
        pbus[0] = 0;
        abus[0] = csr;
        discover_bus(grp, 0, pbus, abus);	// Find PCI/PCI bridges (BUSn)

        //--- Discover non PCI/PCI resources
	//--- devices on same bus are stored together
        for(bus=0; bus<nbus; bus++) for(dev=0; dev<PCI_DEVMAX; dev++)
        {
                u16 tmp;

		if(bus+dev == 0) continue;	// skip PCIhost
                pci_cnvParCsr(bus, dev, 0, &csr);
                if( !csr || !PCI_CSRHDR0(csr) ) continue;
		pci_grpAdd(grp, csr);
		pci_devScan(csr);	// addon device
                nadd++;
                if( PCI_MULTIFN(csr) )
                        for(fun=1; fun<PCI_FUNMAX; fun++)
                        {
                                pci_cnvParCsr(bus, dev, fun, &csr);
                                if(csr)
                                {
					pci_grpAdd(grp, csr);
					pci_devScan(csr);	// addon device
                                        nadd++;
                                }
                        }
        }
	//--- Now all devices have been discovered (size is known)
	//--- Reorder devices by size, bus and type of resource wanted...
	//--- PCI bridges goes at bottom of (size, bus) sets...
	memset(n, 0x00, sizeof(n));
	memset(a, 0xFF, sizeof(a));
	memset(s, 0x00, sizeof(s));

	//-- map requests from device-sorted to type-sorted
	for(i=0; grp[i].csr; i++)
	{
		// Find proper BUS (bus) for this DEVICE (i)
                pci_cnvCsrPar(grp[i].csr, &bus, &dev, &fun);
		for(j=0; j<PCI_GRPMAX; j++)
		{
			u32 *addr;

			// Add all requests to right BUS,TYPE address
			for(k=0; (addr=grp[i].addr[j][k]); k++)
			{
				d[bus][j][ n[bus][j] ] = i;
				p[bus][j][ n[bus][j] ] = addr;
				s[bus][j][ n[bus][j] ] = pci_getSize(addr);
				n[bus][j]++;
			}
#ifdef DEPURACION_E
		if(k)
			printk("[%i,%i] %i request(s) type%i. total %i\n",
				bus, dev, k, j, n[bus][j]);
#endif
		}
	}
	//--- Bubblesort all resources (big to small)
	//--- This will keep PCI map small enough
	for(bus=0; bus<nbus; bus++) for(j=0; j<PCI_GRPMAX; j++)
	{
		u32 kmax = n[bus][j];
		int x, y;

		s[bus][j][kmax] = 0;
		if(!kmax) continue;

		// simple bubblesort
		for(x=kmax-1; x>=0; x--) for(y=0; y<x; y++)
			if(s[bus][j][y] < s[bus][j][y+1])
			{
				register u32 tmp, *ptmp;

				tmp = s[bus][j][y];
				s[bus][j][y] = s[bus][j][y+1];
				s[bus][j][y+1] = tmp;

				tmp = d[bus][j][y];
				d[bus][j][y] = d[bus][j][y+1];
				d[bus][j][y+1] = tmp;

				ptmp = p[bus][j][y];
				p[bus][j][y] = p[bus][j][y+1];
				p[bus][j][y+1] = ptmp;
			}
	}	

	//--- In my opinion, is very difficult to fiddle with
	//--- PCI bridge address masking and maintaining restrictions
	//--- for assignments in compatible-memory (ISA) for low
	//--- 1Mbyte, so I won't assure proper assignment for
	//--- those kind of devices not in BUS0. If you have better
	//--- ideas, please throw a light on it!!

	// We have made array of resources requested for "sizes" s[][]
	// We also have argument mf[] with "resource ranges"
	for(j=0; j<PCI_GRPMAX; j++)
	{
		// For PCI/BRIDGE masking, roundup must be:
		//   * Pad 20 bits (1Mbyte) for RAM/MEM/ROM
		//   * Pad 12 bits (4Kbyte) for I/O
		// For PCI/DEVICE decoding, roundup must be:
		//   * Pad 11 bits (2Kbyte) for ROM
		//   * Pad  4 bits(16 byte) for RAM/MEM
		//   * Pad  2 bits( 4 byte) for I/O -> recommended 16
		// ISA decoding (I mean, <1Mb legacy devices)
		//   * No bridge cross is allowed (only BUS0 devices)
		//   * Restricted to lower 1Mb
#define ROUNDUP(A,B) (((A+B-1)/B)*B)
#define ROUNDUP_1M(A)	ROUNDUP(A, (1<<20))
#define ROUNDUP_4K(A)	ROUNDUP(A, (1<<12))
#define ROUNDUP_2K(A)	ROUNDUP(A, (1<<11))
#define ROUNDUP_16(A)	ROUNDUP(A, (1<< 4))

		u32 base = mf->min[j];

#ifdef NEVER
		if(j==PCI_GRPROM)
			base = ROUNDUP_2K(base);
		else
			base = ROUNDUP_16(base);
#endif

		for(bus=0; bus<nbus; bus++)
		{
			//--- Assign lower range for bridge resource
			fb[bus][j] = base;
			//--- Calculate end of range for this bus
			for(k=0; s[bus][j][k]; k++)
			{
				if(base & (s[bus][j][k]-1))
				{
					base = ROUNDUP(base, s[bus][j][k]);
					if(k==0) fb[bus][j] = base;
				}
				a[bus][j][k] = PCI_GRPROM ? base|1:base;
				base += s[bus][j][k];
				// Roundup for device decoding...
				if(j==PCI_GRPROM)
					base = ROUNDUP_2K(base);
				else
					base = ROUNDUP_16(base);
			}
			//--- Roundup to bridge boundary...
			if(j==PCI_GRPIO)
				base = ROUNDUP_4K(base);
			else
				base = ROUNDUP_1M(base);
			//--- Assign upper range for bridge resource
			if(fb[bus][j]==base)
			{
				fb[bus][j] = 0;
				fl[bus][j] = 0;
			}
			else
				fl[bus][j] = base - 1;
#ifdef DEPURACION
			printk("bus%i.type%i.assign %08X-%08X\n",
				bus, j, fb[bus][j], fl[bus][j]);
			for(k=0; s[bus][j][k]; k++)
			{
				u32 sz = s[bus][j][k];
				char c;

				if(sz >= (1<<20))	// 1MB
					{ sz >>=20; c    = 'M'; }
				else if(sz >= (1<<10))	// 1KB
					{ sz >>=10; c    = 'K'; }
				else
					{ c    = 'B'; }
			
				printk(" [%i,%2i] %3i%c %s: 0x%08X\n",
					 bus, d[bus][j][k], sz, c,
					 pci_strType(p[bus][j][k]),
					 a[bus][j][k] /* & 0xFFFFFFFE */);
			}
#endif
		}	// for bus in nbus
	}	// for j in GRPDEV

	//--- Finally, assign resources...
        for(i=0; grp[i].csr; i++)
	{
		u32 r[7];

		memset(r, 0xFF, sizeof(r));
		// Find proper BUS (bus) for this DEVICE (i)
                pci_cnvCsrPar(grp[i].csr, &bus, &dev, &fun);
		
		for(j=0; j<PCI_GRPMAX; j++) for(k=0; s[bus][j][k]; k++)
			if(d[bus][j][k] == i)
			{
				int x;
				// Index ('u32' entries) in CSR space
				x = ((u32)p[bus][j][k] & 0xFF) >> 2;

				if(x >  10)	// Past baseaddr zone: rom
					r[6]   = a[bus][j][k];
				else
					r[x-4] = a[bus][j][k];
			}
		pci_devInit(grp[i].csr, r, fb[i+1], fl[i+1]);
	}
	return 1;
}

LIBENTRY int pci_post(u32 ramsize, u32 romsize)
{
	pciFrame_t	mf;
        csr_t	       *csr;

	// ---  PCI resource assignment
	// --- I/O
	// [0x****0000-0x****3FFF] for ISA I/O (PC compat. in SouthBridges)
	// [0x****4000-0x****F000] for PCI I/O
	// --- MEM (1GB because PCI frame in CPU mappings --> eg. in Linux)
	// [0x00000000-0x000FFFFF]   1M for LOW MEM (not possible in BUS > 0)
	// [0x00100000-0x07FFFFFF] 127M for ROM/MEM (PCI) no-pref
	// [0x08000000-0x3FFFFFFF] 768M for RAM (PCI) pref

	//--- I/O
	mf.min[PCI_GRPIO]  = 0x00004000;
	mf.max[PCI_GRPIO]  = 0x0000FFFF;
	//--- MEM
	mf.min[PCI_GRPISA] = 0x00000000;
	mf.max[PCI_GRPISA] = 0x00FFFFFF;
	mf.min[PCI_GRPMEM] = 0x01000000;
	mf.max[PCI_GRPMEM] = 0x07FFFFFF;
	mf.min[PCI_GRPRAM] = 0x08000000;
	mf.max[PCI_GRPRAM] = 0x3FFFFFFF;

	pci_cnvParCsr(0, 0, 0, &csr);
	if(!pci_hostInit(&mf))
		return 0;
#ifdef NEVER
	printk("I/O 0x%08X-0x%08X\n", mf.min[PCI_GRPIO ], mf.max[PCI_GRPIO ]);
	printk("MEM 0x%08X-0x%08X\n", mf.min[PCI_GRPMEM], mf.max[PCI_GRPMEM]);
	printk("RAM 0x%08X-0x%08X\n", mf.min[PCI_GRPRAM], mf.max[PCI_GRPRAM]);
#endif

	return pci_init(&mf);
}

