
/***************************************************************************
    libBIOS - A portable library for developing an OpenSource BIOS
    Copyright (C) 2001 - David Aparicio

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Author can be reached at 'daparic@terra.es'
****************************************************************************/
// 
#include "libBIOS.h"

LIBENTRY int pci_getStrClass(csr_t *csr, char *str)
{
        switch(PCI_CLASS(csr)) {
	case 0x0000: sprintf(str, "Generic device"); break;
	case 0x0101: sprintf(str, "IDE controller"); break;
	case 0x0200: sprintf(str, "Ethernet device"); break;
	case 0x0600: sprintf(str, "Host Bridge"); break;
	case 0x0601: sprintf(str, "ISA/PCI Bridge"); break;
	case 0x0604: sprintf(str, "PCI/PCI Bridge"); break;
//	case 0x0605: sprintf(str, "PCMCIA/PCI Bridge"); break;
//	case 0x0607: sprintf(str, "CardBus/PCI Bridge"); break;
//	case 0x0680: sprintf(str, "Generic Bridge"); break;
	case 0x0B40: sprintf(str, "Coprocessor"); break;
//	case 0x0C00: sprintf(str, "FireWire controller"); break;
	case 0x0C03: sprintf(str, "USB controller"); break;
	case 0x0E00: sprintf(str, "Main Processor"); break;
	default:
		switch(csr->class1)
		{
		case 1: sprintf(str, "Storage device 0x%02X",csr->class2);
			break;
		case 2: sprintf(str, "LAN device 0x%02X", csr->class2);
			break;
		case 3: sprintf(str, "Display device 0x%02X", csr->class2);
			break;
		case 4: sprintf(str, "Multimedia device 0x%02X", csr->class2);
			break;
		case 5: sprintf(str, "Memory device  0x%02X", csr->class2);
			break;
		case 6: sprintf(str, "Bridge device %02X", csr->class2);
			break;
		default:sprintf(str, "PCICLASS 0x%02X:0x%02X",
						csr->class1, csr->class2);
			return 0;
		}
	}
	return 1;
}

LIBENTRY int pci_getStrDevice(csr_t *csr, char *str)
{
	switch(PCI_DEVICE(csr))
	{
	case 0x10111065: sprintf(str, "SA110 / DC21285  "); break;
	case 0x12D51000: sprintf(str, "Type-M DSP       "); break;
	case 0x10222000: sprintf(str, "NIC PCnetIII 79C973    "); break;
	case 0x10110022: sprintf(str, "PCI/PCI bridge i21150  "); break;
	case 0x10B91533: sprintf(str, "PCI/ISA bridge ALi1543 "); break;
	case 0x10B95229: sprintf(str, "IDE controller ALi5229 "); break;
	case 0x10B95237: sprintf(str, "USB controller ALi5237 "); break;
	case 0x10B97101: sprintf(str, "PMU controller ALi7101 "); break;
#ifdef DEBUG
	case 0x0E11AE32: sprintf(str, "TI TLan TNETE100A"); break;
	case 0x10110019: sprintf(str, "Digital DC21142  "); break;
	case 0x10505A5A: sprintf(str, "WinBond 89C???   "); break;
	case 0x10AD0105: sprintf(str, "WinBond 82C105   "); break;
	case 0x10AD0565: sprintf(str, "WinBond 83C565   "); break;
	case 0x10EA2010: sprintf(str, "InterGraph 2010  "); break;
	case 0x10EC8029: sprintf(str, "RealTek 8029     "); break;
	case 0x53338811: sprintf(str, "S3 Trio          "); break;
#endif
	default:
		switch(csr->vendor)
		{
#ifdef DEBUG
		case 0x0E11: sprintf(str, "Texas Instruments"); break;
		case 0x1011: sprintf(str, "Digital Corp.    "); break;
		case 0x1022: sprintf(str, "AMD              "); break;
		case 0x1050: sprintf(str, "WinBond          "); break;
		case 0x10AD: sprintf(str, "WinBond          "); break;
		case 0x10B9: sprintf(str, "AcerLabs         "); break;
		case 0x10EA: sprintf(str, "InterGraph       "); break;
		case 0x10EC: sprintf(str, "RealTek          "); break;
		case 0x5333: sprintf(str, "S3               "); break;
#endif
	        default:
			sprintf(str,"PCIDEV %04X:%04X",csr->vendor,csr->device);
			return 0;
		}
	}
	return 1;
}
