
/***************************************************************************
    libBIOS - A portable library for developing an OpenSource BIOS
    Copyright (C) 2001 - David Aparicio

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Author can be reached at 'daparic@terra.es'
****************************************************************************/
// 
#include "libBIOS.h"
#include <stdarg.h>

const char ___hex[] = "0123456789ABCDEF";

static int numerize(char *s, u32 val, u32 base, s32 align)
{
	char buf[128], *p, *s0;

	s0 = s;
	p = buf; *p++ = 0;
	// *** Convertir a ascii en buffer interno
	if(!val)
		*p++ = '0';
	else while(val)
	{
		*p++ = ___hex[val%base];
		val /= base;
	}
	// *** Paddle con ceros o espacios...
	//if(base==16) align = 8;
	align -= (p-buf)-1;
	while(align-- >0)
		*s++ = base==16 ? '0' : ' ';
	// *** Copiar a zona definitiva
	while(1)
	{
		p--;
		if(!*p) break;
		*s++ = *p;
	}
	return s-s0;
}

LIBENTRY int vsprintf(char *buf, const char *fmt, va_list args)
{
	s32 paddle;
	u32 value;
	char *auxp, *buf0;

	buf0 = buf;
	while(*fmt)
	{
		switch(*fmt)
		{
		case '%':
			fmt++;
			if(*fmt=='%')
			{
				*buf++ = *fmt++;
				break;
			}
			paddle = 0;
			while(isdigit(*fmt))
			{
				paddle = paddle*10 + (*fmt-'0');
				fmt++;
			}
			switch(tolower(*fmt))
			{
			case 's':	// STRING
				auxp  = (char *)va_arg(args, char *);
				paddle -= strlen(auxp);
				while(paddle-- > 0) *buf++ = ' ';
				if(!auxp)
				{
					strcpy(buf, "(void)");
					buf += 6;
				}
				else
					while(*auxp) *buf++ = *auxp++;
				break;
			case 'd':	// DECIMAL INTEGER
			case 'u':	// DECIMAL INTEGER
			case 'i':	// DECIMAL INTEGER
				value  = (u32 )va_arg(args, u32);
				buf += numerize(buf, value, 10, paddle);
				break;
			case 'p':	// HEXA INTEGER32 (ZERO PADDED)
			case 'x':	// HEXA INTEGER32 (ZERO PADDED)
				value  = (u32 )va_arg(args, u32);
				buf += numerize(buf, value, 16, paddle);
				break;
			case 'c':	// CHAR
				*buf++ = (char)va_arg(args, u32);
				break;
			default:
				break;
			}
			fmt++;
			break;
		case '\n':
			*buf++ = '\r';
			*buf++ = *fmt++;
			break;
		default:
			*buf++ = *fmt++;
			break;
		}
	}
	*buf = 0;
	return buf - buf0;
}

LIBENTRY int sprintf(char *str, const char *format, ...)
{
	int len;
	va_list ap;
	va_start(ap, format);
	len = vsprintf(str, format, ap);
	va_end(ap);
	return len;
}
