#include "load.h"
#include "dc21285.h"	// OFS_MBOX2

extern void _start(void);
#define IMG_HEADER(I)	((*(mbA_t *)_start)+(I))
#define IMG_ADDR(I)	((u32)_start + IMG_HEADER(I)->offset)
#define IMG_ARCH(I)	(IMG_HEADER(I)->arch)
#define IMG_EOF(I)	(IMG_HEADER(I)->offset == 0)
#define IDX_OF(SFS)	((SFS) - (mbE_t *)_start)

static void copy_segment(u8 *des, u8 *ori, u32 sze)
{
	if(ori == des || !sze) return;
	printk("Copying 0x%08X -> 0x%08X (%i bytes)\n", ori, des, sze);
	memcpy(des, ori, sze);
}

static void copy_image(u32 addr, mbE_t *img)
{
	u8 *ptr = (u8 *)IMG_ADDR(IDX_OF(img));

	copy_segment(addr+img->baseC, ptr,              img->sizeC);
	copy_segment(addr+img->baseD, ptr + img->sizeC, img->sizeD);
}

static void do_exec(u32 ramaddr, u32 memaddr, mbE_t *img)
{
	int r;

	printk("Binary signature %s detected\n", img->label);
	switch(img->arch)
	{
	case SFS_ARCH_SA110:
		if(ramaddr==0)	// hostmode
		{
			copy_image(0, img);
			exit(img->entry);	// JUMP
		}
		else		// addon mode
		{
			copy_image(ramaddr, img);
			printk("Addon EBSA285 (not implemented)\n");
		}
		break;
	case SFS_ARCH_DSP:
		break;
	default:
		printk("panic - unknown device %08X run (not implemented)\n",
			img->arch);
		while(1) {}
		break;
	}
}

static int compatible(u32 pciid, u32 arch)
{
	switch(pciid)
	{
	case ID_EBSA285:	return (arch==SFS_ARCH_SA110) ? 1 : 0;
	case ID_DSPMCA2:	return (arch==SFS_ARCH_DSP  ) ? 1 : 0;
	default: return 0;
	}
}

int entry(env_t *env)
{
	csrGrp_t grp;
	u32      i, j;

	printk("=(2)========================================\n");
	printk("            [FW"DATE" LOADER]\n");
	printk("============================================\n");
	// --- Identificar NUMERO de CPUs y TIPO
	// Check if we have been called by post or not
	// If so, then initialize PCI bus...
	if(!env->warmreset)
	{
		printk("--- Initializing PCI subsystem\n");
		pci_post(env->ramsize, env->romsize);
		env->warmreset = 1;
	}
	IniHeap(ADDR_MCB, SZ_32M);

	pci_grpScan(grp);
	//--- Load all images on daughercards
	for(i=1; !IMG_EOF(i); i++)
	{
		//--- Search device for this image
		if(SFS_NOPCI(IMG_ARCH(i)))	// Non PCI device
			do_exec(IMG_ADDR(i), 0, IMG_HEADER(i));
		else for(j=1; grp[j].csr; j++)		// Default PCI search
			if(compatible(PCI_DEVICE(grp[j].csr), IMG_ARCH(i)))
			{
				u32 ram, mem;

				ram = pci_getAddr(grp[j].addr[PCI_GRPRAM][0]);
				mem = pci_getAddr(grp[j].addr[PCI_GRPMEM][0]);
				do_exec(pci_bus2ram(ram),
					pci_bus2ram(mem), IMG_HEADER(i));
			}
	}
#define IS_EXECUTABLE(I) (IMG_HEADER(I)->baseC < (u8 *)ADDR_POST)
	//--- Now load image on this host (skip "flash" images for this host)
	for(i=1; !IMG_EOF(i); i++)
		if(compatible(PCI_DEVICE(grp[0].csr), IMG_ARCH(i))
		   && IS_EXECUTABLE(i))
			do_exec(0, 0, IMG_HEADER(i));

	printk("No image for CPU0? - hang\n");
	while(1) {}
	return 0;
}
