#include "mon.h"
#include "dc21285.h"
#include "ali1543.h"

//************** MEMORY READ/WRITE **********************
static void access(int bytesize, u32 addr, u32 addrf, u32 val)
{
	int is_write;

	if(bytesize<0)
	{
		is_write = 1;
		bytesize = -bytesize;
	}
	else
		is_write = 0;
	while(addr < addrf)
	{
		unsigned i0, i1;
		u32 addr0;

		printk("%08X: ", addr);
		i0 = 0;
		i1 = 16 < addrf-addr ? 16 : addrf-addr;
			
		addr0 = addr;
		while(i0 < i1)
		{
			switch(bytesize)
			{
			case 1: if(is_write)
					*(volatile u8  *)addr = val&0x000000FF;
			        printk("%02X ", *(volatile  u8 *)addr);
				break;
			case 2: if(is_write)
					*(volatile u16 *)addr = val&0x0000FFFF;
			        printk("%04X ", *(volatile u16 *)addr);
				break;
			case 4: if(is_write)
					*(volatile u32 *)addr = val&0xFFFFFFFF;
			        printk("%08X ", *(volatile u32 *)addr);
				break;
			}
			i0   += bytesize;
			addr += bytesize;
		}
		if(bytesize==1)
		{
			while(i0++<16)		// padding de espacios
				printk("   ");
			i0 = 0;
			printk("  ");
			addr = addr0;		// vuelve a leer la memoria
			while(i0<i1)
			{
				u8 c = *(volatile u8 *)addr;
				if(c<' ') c = '.';
				if(i0==8) printk(" ");
				printk("%c", c);
				i0++; addr++;
			}
		}
		printk("\n");
	}
}

void rdmem(u16 tipo, u16 nelem, u32 elem[], u16 bytesize)
{
	int i;

	switch(tipo)
	{
	case 1: // <addr0>
		access(bytesize, elem[0], elem[0]+bytesize, 0);
		break;
	case 2: // <addr0>-<addr1>
		access(bytesize, elem[0], elem[1], 0);
		break;
	case 3: // <addr0>[<size>]
		access(bytesize, elem[0], elem[0]+bytesize*elem[1], 0);
		break;
	case 4: // <addr0>,<addr1>, ... <addrn>
		for(i=0; i<nelem; i++)
			access(bytesize, elem[i], elem[i]+bytesize, 0);
		break;
	default:
		printk("??? r[bwl] <rango>\n");
		break;
	}
}

void wrmem(u16 tipo, u16 nelem, u32 elem[], u32 val, u16 bytesize)
{
	int i;

	switch(tipo)
	{
	case 1: // <addr0>
		access(-bytesize, elem[0], elem[0]+bytesize, val);
		break;
	case 2: // <addr0>-<addr1>
		access(-bytesize, elem[0], elem[1], val);
		break;
	case 3: // <addr0>[<size>]
		access(-bytesize, elem[0], elem[0]+bytesize*elem[1], val);
		break;
	case 4: // <addr0>,<addr1>, ... <addrn>
		for(i=0; i<nelem; i++)
			access(-bytesize, elem[i], elem[i]+bytesize, val);
		break;
	default:
		printk("??? w[bwl] <rango> <valor>\n");
		break;
	}
}

//************** DEVICE READ/WRITE **********************
void rapp(void)
{
	printk("PPA: 0x%02X\n", *PCIPORT(PP0_ECPADR));
}

void rdpp(void)
{
	printk("PPD: 0x%02X\n", *PCIPORT(PP0_ECPDAT));
}

void wapp(u32 val)
{
	*PCIPORT(PP0_ECPADR) = val & 0xFF;
	printk("PPA: 0x%02X\n", val & 0xFF);
}

void wdpp(u32 val)
{
	*PCIPORT(PP0_ECPDAT) = val & 0xFF;
	printk("PPD: 0x%02X\n", val & 0xFF);
}

#define RTC_ADDR(AD) *PCIPORT(0x70) = AD
#define RTC_READ     *PCIPORT(0x71)
#define RTC_WRITE(V) *PCIPORT(0x71) = V

int rtc_init(void)
{
	RTC_ADDR(13);	// registro D
	if((RTC_READ&0x80)==0) return -1;	// Bateria descargada!!
	RTC_ADDR(11);	// registro B
	RTC_WRITE(0x07);	// modo binario, 24 horas, daylight saving
	RTC_ADDR(10);	// registro A
	RTC_WRITE(0x30);	// banco 1
	return 0;
}

// La memoria del RTC la hemos dividido en:
// at x00,   1 byte , seconds BANCO0[x00] o BANCO1[x00]
// at x01,   1 byte , minutes BANCO0[x02] o BANCO1[x02]
// at x02,   1 byte , hours   BANCO0[x04] o BANCO1[x04]
// at x03,   1 byte , day     BANCO0[x07] o BANCO1[x07]
// at x04,   1 byte , month   BANCO0[x08] o BANCO1[x08]
// at x05,   1 byte , year    BANCO0[x09] o BANCO1[x09]
// at x06,   8 bytes, serial  BANCO1[x40..x47]
// at x0E,  50 bytes, BANCO0[x0E..x3F] o BANCO1[x0E..x3F]
// at x40,  64 bytes, BANCO0[x3F..x7F]
// at x80, 128 bytes, EXTENDED (registro BANCO1[x50] y BANCO1[x53])

u8   rtc_read(u8 offs)
{
	switch(offs)
	{
	case 0x00: RTC_ADDR(0x00); return RTC_READ;
	case 0x01: RTC_ADDR(0x02); return RTC_READ;
	case 0x02: RTC_ADDR(0x04); return RTC_READ;
	case 0x03: RTC_ADDR(0x07); return RTC_READ;
	case 0x04: RTC_ADDR(0x08); return RTC_READ;
	case 0x05: RTC_ADDR(0x09); return RTC_READ;
	case 0x06:
	case 0x07:
	case 0x08:
	case 0x09:
	case 0x0A:
	case 0x0B:
	case 0x0C:
	case 0x0D:
		RTC_ADDR(10);	// registro A
		RTC_WRITE(0x30);	// banco 1
		RTC_ADDR(0x40 + offs-0x06);
		return RTC_READ;
	default:
		if(offs<0x40)
		{
			RTC_ADDR(offs);	// offs entre 0x0E y 0x3F
			return RTC_READ;
		}
		if(offs<0x80)
		{
			RTC_ADDR(10);
			RTC_WRITE(0x20);	// banco 0
			RTC_ADDR(offs);	// offs entre 0x40 y 0x7F
			return RTC_READ;
		}
		if(offs<0x100)
		{
			RTC_ADDR(10);
			RTC_WRITE(0x30);	// banco 1
			RTC_ADDR(0x50);		// extended ADDRESS
			RTC_WRITE(offs-0x80);
			RTC_ADDR(0x53);		// extended DATA
			return RTC_READ;
		}
		return 0xFF;
	}
}

void rtc_write(u8 offs, u8 val)
{
	switch(offs)
	{
	case 0x00: RTC_ADDR(0x00); RTC_WRITE(val); return;
	case 0x01: RTC_ADDR(0x02); RTC_WRITE(val); return;
	case 0x02: RTC_ADDR(0x04); RTC_WRITE(val); return;
	case 0x03: RTC_ADDR(0x07); RTC_WRITE(val); return;
	case 0x04: RTC_ADDR(0x08); RTC_WRITE(val); return;
	case 0x05: RTC_ADDR(0x09); RTC_WRITE(val); return;
	case 0x06:
	case 0x07:
	case 0x08:
	case 0x09:
	case 0x0A:
	case 0x0B:
	case 0x0C:
	case 0x0D:
		RTC_ADDR(10);	// registro A
		RTC_WRITE(0x30);	// banco 1
		RTC_ADDR(0x40 + offs-0x06);
		RTC_WRITE(val);
		return;
	default:
		if(offs<0x40)
		{
			RTC_ADDR(offs);	// offs entre 0x0E y 0x3F
			RTC_WRITE(val);
			return;
		}
		if(offs<0x80)
		{
			RTC_ADDR(10);
			RTC_WRITE(0x20);	// banco 0
			RTC_ADDR(offs);	// offs entre 0x40 y 0x7F
			RTC_WRITE(val);
			return;
		}
		if(offs<0x100)
		{
			RTC_ADDR(10);
			RTC_WRITE(0x30);	// banco 1
			RTC_ADDR(0x50);		// extended ADDRESS
			RTC_WRITE(offs-0x80);
			RTC_ADDR(0x53);		// extended DATA
			RTC_WRITE(val);
			return;
		}
		return;
	}
}
