#include "mon.h"

// Extrae una palabra (elimina tab, espacios) de 's' y la deja en 'w'
// SI se trata de un caracter especial (';', '{', '}', ...) toma solo ese
int getword(char **s, char *w)
{
	char *w0 = w;
	char *p = *s;

	while(*p && *p<=' ') p++;	// quita leading blanks
	if(isalnum(*p) || *p=='.')
		while(*p && (isalnum(*p) || *p=='.'))
			 *w++ = *p++;	// copia contenidos
	else if(*p)	 // Caracter especial, copiar solo ese
		*w++ = *p++;
	*w = 0;
	while(*p && *p<=' ') p++;	// quita trailing blanks
	*s = p;
	return (w-w0);		// retorna si w contiene algun caracter...
}

static u32 getnumber0(char **s, u32 base)
{
	u32 num;
	char *p = *s;
	for(num=0; *p; p++)
	{
		const char hex[] = "0123456789ABCDEF";
		char *c;

		c = strchr(hex, toupper(*p));
		if(!c || (c-hex>=base)) break;
		num = base*num + (c-hex);
	}
	*s = p;
	return num;
}

int getnumber(char **s, u32 *n)
{
	char *p0, *p = *s;
	u32  num, base;

	while(*p && *p<=' ') p++;	// quita leading blanks
	p0 = p;
	if(*p=='0' && toupper(*(p+1))=='X')
		{ base = 16; p += 2; }
	else if(*p=='0' && toupper(*(p+1))=='B')
		{ base = 2;  p += 2; }
	else if(*p=='0' && *(p+1)>='0' && *(p+1)<='7')
		{ base = 8;  p += 1; }
	else
		base = 10;
	num =  getnumber0(&p, base);
	if(p0==p) p0 = NULL;	// no hay numero!!
	else
	while(base==10 && *p=='.' && num<256)	// Puede ser una IP
	{
		char *p1;
		u8  ipA, ipB, ipC, ipD;
		u32 tmp;
		
		ipA = num; p++;	p1 = p; // Saltar punto
		tmp = getnumber0(&p, base);
		if(p1==p || tmp>255 || *p!='.') break;  // Error
		ipB = tmp; p++;	p1 = p; // Saltar punto
		tmp = getnumber0(&p, base);
		if(p1==p || tmp>255 || *p!='.') break;  // Error
		ipC = tmp; p++;	p1 = p; // Saltar punto
		tmp = getnumber0(&p, base);
		if(p1==p || tmp>255)            break;  // Error
		ipD = tmp;
		num = ipA<<24 | ipB<<16 | ipC<<8 | ipD;
		break;
	}
	while(*p && *p<=' ') p++;	// quita trailing blanks
	*s = p;
	*n = num;
	return p0 ? 1 : 0;
}

int getrange(char **s, u16 *n, u32 ln[])
{
	// <num>: Numero simple (return 1)
	// <num>-<num>: Rango   (return 2)
	// <num>[<num>]: Rango  (return 3)
	// <num>, <num> ...: Lista (return 4, en n entra maximo en array y
	//				sale numero elementos rellenos)
	char *p, w[128];
	u16 ctr, max = *n;	

	if(!getnumber(s, ln)) return 0;
	p = *s;
	if(getword(&p, w))
	{
		if(w[1]==0) switch(w[0])
		{
		case '[':
			*s = p;
			if(!getnumber(s, ln+1)) return 0;
			if(!getword(s, w)) return 0;
			if(w[0]!=']' || w[1]!=0) return 0;
			*n = 2;
			return 3;
		case '-':
			*s = p;
			if(!getnumber(s, ln+1)) return 0;
			*n = 2;
			return 2;
		case ',':
			*s = p;
			if(!getnumber(s, ln+1)) return 0;
			ctr = 2;
			while(ctr < max)
			{
				p = *s;
				if(!getword(&p, w)) break;
				if(w[0]!=',' || w[1]!=0) break;
				*s = p;
				if(!getnumber(s, ln+ctr)) return 0;
				ctr++;
			}
			*n = ctr;
			return 4;
			
		}
	} // getword(, [ -)
	*n = 1;
	return 1;
}

int do_null(char **s)
{
	char *p, w[1024];

	p = *s;
	while(1)
	{
		int r = getword(&p, w);

		if(!r || (w[0]==';' && w[1]==0)) break;
		*s = p;
	}
	return 1;
}

int do_quit(char **s)
{
	return -1;
}

int do_begin(char **s)
{
	return 100;
}

int do_end(char **s)
{
	return 101;
}

int do_void(char **s)
{
	// El punto y coma...
	return 0;
}

int do_help(char **s)
{
	int i;

	for(i=0; cmds[i].name; i++)
		if(cmds[i].name[1]) printk("??? <%s>\n", cmds[i].name);
	return 0;
}

int do_map(char **s)
{
	u32 n;
	char *p, w[256];

	p = *s;
	if(getnumber(&p, &n))
	{
		*s = p;
		cmd_map((int) n);
		return 0;
	}
	p = *s;
	cmd_map((int)-1);
	return 0;
}

int do_pci(char **s)
{
	u32 n;
	char *p, w[256];

	p = *s;
	if(getnumber(&p, &n))
	{
		*s = p;
		cmd_pci((int) n);
		return 0;
	}
	p = *s;
	cmd_pci((int)-1);
	return 0;
}

int do_lock(char **s)
{
	printk("You'll LOCK the bootsector FOREVER - press 'l' to proceed\n");
	if(getc()=='l')
		rom_lock(ADDR_POST);
	else
		printk("Aborted\n");
}

int do_test(char **s)
{
	return 0;
}

int do_sfs(char **s)
{
	mbE_t *mb;
	u32  n, r;
	u8  *addr;
	char w[256];

	if(!getword(s, w))
	{
		cmd_sfs('O', 0, &addr);
		cmd_sfs('A', 0, &addr);
		return 0;
	}
	w[1] = tolower(w[1]);
	if(w[1]!='a' && w[1]!='o') return 0;
	if(*(*s)==':') (*s)++;
	r = getnumber(s, &n);

	mb = cmd_sfs(r ? w[1] : toupper(w[1]), n, &addr);
	if(mb)
	{
		if(mb->type == SFS_TYPE_LOAD)	// Flash current
			burn(mb, addr);
		else
			copy(mb, addr);
	}
	return 0;
}

int do_jump(char **s)
{
	u32 n;

	if(getnumber(s, &n))
	{
		printk("Jump to 0x%08X\n", n);
		exit(n);
	}
	return 0;
}

int do_read(char **s)
{
	char w[128];
	u32 nums[32];
	u16 code;
	u16 num = 32;

	if(!getword(s, w))
		w[0] = 0;
	switch(toupper(w[0]))
	{
	case 'B': code = getrange(s, &num, nums);
		  rdmem(code, num, nums, 1);
		  break;
	case 'W': code = getrange(s, &num, nums);
		  rdmem(code, num, nums, 2);
                  break;
	case 'L': code = getrange(s, &num, nums);
		  rdmem(code, num, nums, 4);
                  break;
	case 'A': rapp();      break;
	case 'D': rdpp();      break;
	default:
	error:
		printk(	"??? r[bwl] <addresslist>\n"
			"??? r[ad]\n");
		return 0;
	}
	return 1;
}

int do_write(char **s)
{
	char w[128];
	u32 nums[32];
	u16 num = 32;
	u16 code;
	u32 val;

	if(!getword(s, w))
		w[0] = 0;
	switch(toupper(w[0]))
	{
	case 'B': code = getrange(s, &num, nums);
		  if(!getnumber(s, &val)) code = 0;
		  wrmem(code, num, nums, val, 1);
                  break;
	case 'W': code = getrange(s, &num, nums);
		  if(!getnumber(s, &val)) code = 0;
		  wrmem(code, num, nums, val, 2);
                  break;
	case 'L': code = getrange(s, &num, nums);
		  if(!getnumber(s, &val)) code = 0;
		  wrmem(code, num, nums, val, 4);
                  break;
	case 'A': if(!getnumber(s, &val)) code = 0;
                  wapp(val);
                  break;
	case 'D': if(!getnumber(s, &val)) code = 0;
                  wdpp(val);
                  break;
	default:
	error:
		printk(	"??? w[bwl] <addresslist> <value>\n"
			"??? w[ad] <value>\n");
		return 0;
	}
	return 1;
}

const cmds_t cmds[] = {
#ifdef DEBUG
		{ "debug",	do_debug,	0    },
#endif
		{ "help",	do_help,	1    },
		{ "jump",	do_jump,	1    },
		{ "map",	do_map,		1    },
		{ "pci",	do_pci,		1    },
		{ "sfs",	do_sfs,		1    },
		{ "quit",	do_quit,	1    },
		{ "read",	do_read,	1    },
		{ "romlock",	do_lock,	1    },
		{ "test",	do_test,	1    },
		{ "write",	do_write,	1    },
		{ "?",		do_read,	1    },
		{ "=",		do_write,	1    },
		{ "{",		do_begin,	1    },
		{ "}",		do_end,		1    },
		{ ";",		do_void,	1    },

		{ NULL,		do_null },
		};

static fun_t search(char *c)
{
	int i;

	for(i=0; c[i]; i++) if(isupper(c[i])) c[i] |= 0x20;
	for(i=0; cmds[i].name; i++)
	{
		char *s, *d;

		s = cmds[i].name;
		d = c;

		while(*s && *d)
			if(*s==*d)
				s++,d++;
			else
				break;
		if(*s==0 || *d==0)
			return cmds[i].fun;
	}
	return FNUL;
}

int parseline(char *s)
{
	char w[256];
	fun_t  cmd;
	int r = 0;

	while(s && *s)
	{
		if(getword(&s, w))	// Procesar comando
		{
			if(!(cmd = search(w)))	// No existe el comando
			{
				if(w[0]<' ') return 0;	// Fin de batch??
				r = do_null(&s);
			}
			else
				r = cmd(&s);	// Ejecutar comando
		}
	}
	return r;
}

#ifdef NEVER
static char *nm_batch = NULL;

char *is_batch(void)
{
	return nm_batch;
}

int script(char *file, u32 addr, u32 size)
{
	int r;
	char *s = (char *)addr;

	s[size-1] = 0;	// Asegura fin de batch-file
	nm_batch = file;
	r = parseline(s);
	nm_batch = NULL;
	return r;
}
#endif
