#include "mon.h"
#include "dc21285.h"
#include "ali1543.h"

static char *strirq(int slot, int irq)
{
#define IRQ_PCI	16+12
#define IRQ_IN0	16+5
#define IRQ_IN1	16+6
#define IRQ_IN2	16+7
#define IRQ_IN3	16+8
	const int irqmap[] = { IRQ_PCI, IRQ_IN0, IRQ_IN1, IRQ_IN3 };
	static char str[64];

	switch(irq)
	{
	case 0  : return "";
	case 255: return "**";
	default :
		if(irq & 0x80)
			sprintf(str, "%2i (PIC)", irq & 0x7F);
		else switch(slot)
		{
			case  8: sprintf(str, "%2i (INTA)", irqmap[0]); break;
			case  9: sprintf(str, "%2i (INTB)", irqmap[1]); break;
			case 10: sprintf(str, "%2i (INTC)", irqmap[2]); break;
			case 11: sprintf(str, "%2i (INTD)", irqmap[3]); break;
			default: return "";
		}
		break;
	}
	return str;
}

static char *stradr(void *addr0, int shortprint)
{
	int len;
	u32 t, size;
	static char str[128];

	if(!addr0 || addr0==CSRNUL) return NULL;
	t    = pci_getType(addr0);
	size = pci_getSize(addr0);

	len = 0;
	len += sprintf(str+len, "  %s: ", pci_strType(addr0));
	if(size>=1024*1024)
		len += sprintf(str+len, "(%3i MB) ", size>>20);
	else if(size>=1024)
		len += sprintf(str+len, "(%3i KB) ", size>>10);
	else if(size)
		len += sprintf(str+len, "(%3i  B) ", size);
	else
		len += sprintf(str+len, "(???  B) ");
        len += sprintf(str+len, "0x%08X-0x%08X ",
				pci_getAddr(addr0), pci_getAddr(addr0)+size-1);
	if(!shortprint)
	{
		if(pci_getType(addr0)==PCI_GRPIO)
		  len += sprintf(str+len, "[0x%08X-0x%08X] ",
				pci_bus2io(pci_getAddr(addr0)),
				pci_bus2io(pci_getAddr(addr0)+size-1));
		else
		  len += sprintf(str+len, "[0x%08X-0x%08X] ",
				pci_bus2ram(pci_getAddr(addr0)),
				pci_bus2ram(pci_getAddr(addr0)+size-1));
	}
	return str;
}

static void map_scan1(csrGrp_t g, int index)
{
	int i, type;

	for(type=0; type<PCI_GRPMAX; type++) for(i=0; i<PCI_RESMAX; i++)
	{
		u32  *addr = pci_grpGetAddr(g, index, type, i);

		if(addr)
		{
			u32   size;
			char *q;

			size = pci_grpGetSize(g, index, type, i);
			q = stradr(addr, 1);
			if(q)
			{
				char md[128];

				pci_getStrDevice(g[index].csr, md);
				printk("%s [%i] %s\n", q, index, md);
			}
		}
	}
}

void cmd_map(int dev)
{
	int i;
	csrGrp_t grp;

	pci_grpScan(grp);
	if(dev<0)
	{
		for(i=0; grp[i].csr; i++)
			map_scan1(grp, i);
	}
	else
		map_scan1(grp, dev);
}

void cmd_pci(int dev)
{
	int i, j, k;
	csrGrp_t grp;

	pci_grpScan(grp);
	for(i=0; grp[i].csr; i++)
	{
		char *p;
		csr_t *csr;
		u8 bus, slot, fun;
		char md[128], cl[128];

		csr = pci_grpFindIndex(grp, i);
		if(!csr) break;	// ??
		pci_cnvCsrPar(csr, &bus, &slot, &fun);
		p = strirq(slot, PCI_CSRHDR0(csr)
				? csr->u.hdr0.irq
				: csr->u.hdr1.irq);

		if(dev==i)	// Solo un dispositivo, imprimir y salir
		{
			pci_getStrClass (csr, cl);
			pci_getStrDevice(csr, md);
			printk("Dispositivo %i: %s (%s)\n", i, md, cl);
			printk("  Bus %i slot %i funcion %i", bus, slot, fun);
			printk(" (%s)\n", slot==8  ? "ranura 3" :
					  slot==9  ? "ranura 2" :
					  slot==10 ? "ranura 1" :
					             "placa base");
			printk("  CSR: 0x%08X-0x%08X\n", csr, (u32)csr+255);
#ifdef NEVER
			if(PCI_DEVICE(csr)==0x10111065 && i>0)
			{
			printk("  MBX: %08X, %08X, %08X, %08X\n",
				*(u32 *)pci_bus2cpu(*(pci_grpGetAddr(grp, i, PCI_GRPMEM, 0)+0x50)),
				*(u32 *)pci_bus2cpu(*(pci_grpGetAddr(grp, i, PCI_GRPMEM, 0)+0x54)),
				*(u32 *)pci_bus2cpu(*(pci_grpGetAddr(grp, i, PCI_GRPMEM, 0)+0x58)),
				*(u32 *)pci_bus2cpu(*(pci_grpGetAddr(grp, i, PCI_GRPMEM, 0)+0x5C)));
			}
#endif
			for(j=0; j<PCI_GRPMAX; j++)
			{
				u32 *addr;

				for(k=0; (addr=pci_grpGetAddr(grp,i,j,k)); k++)
				{
					char *q = stradr(addr, 0);

					if(q) printk(" %s\n", q);
				}
			}
			if(PCI_CSRHDR1(csr))
			{
				printk("  BUS: [%i, %i-%i]\n",
					csr->u.hdr1.busE,
					csr->u.hdr1.busI,
					csr->u.hdr1.busmax);
				printk("  I/O: 0x%04X-0x%04X\n",
					((csr->u.hdr1.iobase&0xF0)<<8),
					((csr->u.hdr1.iosize&0xF0)<<8)
					|0x0FFF);
				printk("  MEM: 0x%08X-0x%08X\n",
					((csr->u.hdr1.membase&0xFFF0)<<16),
					((csr->u.hdr1.memsize&0xFFF0)<<16)
					|0x000FFFFF);
				printk("  RAM: 0x%08X-0x%08X (prefetchable)\n",
					((csr->u.hdr1.rambase&0xFFF0)<<16),
					((csr->u.hdr1.ramsize&0xFFF0)<<16)
					|0x000FFFFF);
			}
			if(*p) printk("  Lnea IRQ: %s\n", p);
			else   printk("  Lnea IRQ: No tiene\n");
			printk("\n");
			return;
		}
		else if(dev >= 0) continue;

		switch(bus*0x100|slot)
		{
		case 0x0007: printk("%2i: BUS0 slt4     ", i); break;
		case 0x0008: printk("%2i: BUS0 slt3     ", i); break;
		case 0x0009: printk("%2i: BUS0 slt2     ", i); break;
		case 0x000A: printk("%2i: BUS0 slt1     ", i); break;
		case 0x000B: printk("%2i: BUS0 slt0     ", i); break;
		default: if(!bus)
                             printk("%2i: BUS0 id%02i     ",  i, slot);
                         else
                             printk("%2i: BUS%i slt%i     ", i, bus, slot);
			 break;
		}
		pci_getStrDevice(csr, md);
		printk("%s     ", md);

		if(*p) printk(" IRQ %s\n", p);
		else   printk("\n");
	}
	if(dev >= 0)
		printk("Dispositivo %i no encontrado\n", dev);
	else
		printk("\n");
}

