#include "mon.h"

static void mb_entry(void *code, mbE_t *d, u32 indent, u32 idx)
{
	printk("%2i.%08X.%08X: ", idx, code, crc32(code, d->sizeC));
	while(indent)
	{
		putc(' ');
		indent--;
	}
	printk("%08X(%6i) %08X(%6i) %s/%s - %s\n",
		d->baseC, d->sizeC,
		d->baseD, d->sizeD,
		d->type==SFS_TYPE_POST
			? "POST"
			: d->type==SFS_TYPE_LOAD
				? "LOAD"
				: "EXEC",
		d->arch==SFS_ARCH_SA110
			? "ARM"
			: d->arch==SFS_ARCH_DSP
				? "DSP"
				: "LCA",
		d->label);
}

static u32       count;
static mbE_t *mbE;
static u8       *sfsaddr;

static void mb_array(u32 key, u32 type, u8 *code, u32 indent)
{
	u32 j;
	mbA_t *mbA = (mbA_t *)code;

	switch(type)
	{
	case SFS_TYPE_POST:
		if(!key)
			mb_entry(code, (*mbA)+0, indent, count);
		else if(count == key)
		{
			mbE  = (*mbA)+0;
			sfsaddr = code;
		}
		count++;
		mbA = (mbA_t *)(*mbA)[0].baseD;
		if(IMG_LOAD(mbA))
			mb_array(key, SFS_TYPE_LOAD,
				(u8 *)mbA,
				indent+1);
		break;
	case SFS_TYPE_LOAD:
		if(!key)
			mb_entry(code, (*mbA)+0, indent, count);
		else if(count == key)
		{
			mbE  = (*mbA)+0;
			sfsaddr = code;
		}
		count++;
		for(j=1; (*mbA)[j].offset; j++)
		{
			u32 type = (*mbA)[j].type;

			if(type==SFS_TYPE_POST || type==SFS_TYPE_LOAD)
				mb_array(key, type,
					code + (*mbA)[j].offset,
					indent+1);
			else
			{
				if(!key)
					mb_entry(code + (*mbA)[j].offset,
						(*mbA)+j, indent+1, count);
				else if(count == key)
				{
					mbE  = (*mbA)+j;
					sfsaddr = code + (*mbA)[j].offset;
				}
				count++;
			}
		}
		break;
	default:
		break;
	}
}

static void show(u32 key, void *code)
{
	mbA_t *mbA = (mbA_t *)code;

	count = 1;
	mb_array(key, (*mbA)[0].type, code, 0);
}

mbE_t *cmd_sfs(char c, u32 n, u8 **addr)
{
	mbE  = NULL;
	*addr = sfsaddr = NULL;

	switch(c)
	{
	case 'A':
		printk("--RAM SFS:\n");
	case 'a':
		show(c=='a' ? n : 0, (void *)ADDR_LOAD);
		break;
	case 'O':
		printk("--ROM SFS:\n");
	case 'o':
		show(c=='o' ? n : 0, (void *)ADDR_POST);
		break;
	default:
		break;
	}
	*addr = sfsaddr;
	return mbE;
}
