#include "emOS.h"

static void *trigger;

#define LEN 10

void Sync0(u32 msec)
{
	if(trigger)
	{
		SIGNAL(trigger, LEN);
		trigger = NULL;
	}
}

void Task0(pid_t pid)
{
	u8 *sgn = NULL;

	trigger = NULL;
	// Get a new MCB for signaling
	sgn = NEW(1);
	printk("TaskA: Ready (SGNL is %08X)\n", sgn);
  	while(1)
	{
		u8 *msg;

		if((msg = RECEIVE(TM_NOTIMEOUT, FL_ANY)))
		{
			printk("TaskA: Recv %08X ", msg);
			if(IS_STREAM(msg))
			{
				printk("[STRM]\n");
				trigger = sgn;
			}
			else if(IS_SIGNAL(msg))
			{
				u8 *aux;

				printk("[SGNL]\n");
				aux = NEW(1);
				printk("TaskA: Send %08X [COMM]\n", aux);
				if(CONTROL(aux, 2, LEN, 5000)==R_OK)
					printk("TaskA: Recv %08X-RESP\n", aux);
				else
					printk("TaskA: Recv %08X-TIME\n", aux);
				FREE(aux);
			}
			else
				printk("[????]\n");
			FREE(msg);
		}
		else
			printk("TaskA: SysCall Error\n");
	}
}

void Task1(pid_t pid)
{
	printk("TaskB: Ready\n");
	while(1)
	{
		u8 *msg;

		if((msg = RECEIVE(2000, FL_ANY)))
		{
			printk("TaskB: Recv %08X [%s]\n",msg,
				IS_CONTROL(msg) ? "COMM" : "????");
			REPLY(msg, LEN);
		}
		else if(_errno==R_EAGAIN)
		{
			printk("\nTaskB: Timeout\n");
			msg = NEW(1);
			printk("TaskB: Send %08X [STRM]\n", msg);
			STREAM(msg, 1, LEN);
		}
		else
			printk("TaskB: SysCall Error\n");
	}
}

const stask_t tasks[] = {
	// Higher number is better priority (max MAX_PRIO-1)
	// If you put same priority for task0 and task1, "Ready"
	// messages for this example will mix on your console
	// (round robin policy).

	// ini_proc    poll_tics poll_proc prio   cpu
	// ----------  -----    ---------- -----  -----
//--- CPU0
/*[00]*/{  USR0asyn,      0,    NULL,    0,     0  },
/*[01]*/{  Task0,   	  5,    Sync0,   2,     0  },
/*[02]*/{  Task1,         0,    NULL,    1,     0  }, 
	{  NULL,          0,    NULL }
	};
