#include "libBIOS.h"

#define	SOH		0x01
#define	STX		0x02
#define	EOT		0x04
#define	ENQ		0x05
#define	ACK		0x06
#define	LF		0x0a
#define	CR		0x0d
#define	XON		0x11
#define	XOFF		0x13
#define	NAK		0x15
#define	CAN		0x18

static u32 chkROM(void)
{
	u32 *pos0 = (u32 *)ADDR_POST;
	u32 *pos1 = pos0 + (SZ_128K>>2);

	while((*pos0 != *pos1) && (pos1-pos0 < (SZ_16M>>2)))
		pos1 += (SZ_128K>>2);
	return (pos1-pos0) << 2;
}

static void copy(void *des, void *ori, u32 sze)
{
	if(ori == des || !sze) return;
	printk("Copying 0x%08X -> 0x%08X (%i bytes)\n", ori, des, sze);
	memcpy(des, ori, sze);
}

static u32 moveImage(u32 addr)
{
	mbA_t  *i;
#define SLOT(X) ((*i)[X])

	// Load 2nd/3rd image (manage XMODEM/FLASH detection)
	i = (void *)addr;
	if(IMG_LOAD(i))
	{
		printk("Loader signature %s detected\n", SLOT(0).label);
		//--- If we are on flash, move to download zone
		if(addr == ADDR_ROMLOAD)
			copy(SLOT(0).baseC, i, SLOT(0).sizeC);
		copy(SLOT(0).baseD, (char *)i + SLOT(0).sizeC, SLOT(0).sizeD);

		// Return entry point for this image
		return (u32)SLOT(0).entry;
	}
	// No magic image in flash -> load from XMODEM
	return 0xFFFFFFFF;
}

static int getw(char *c, u32 timeout)
{
	wait(timeout, 1);	// Interruptable wait...
	if(keysleft())
	{
		*c = getc();
		return 0;
	}
	else
		return -1;
}

static void send_nak(void)
{
	while(1)
	{
		wait(1000, 1);
		if(keysleft())
			getc();
		else
			break;
	}
	putc(NAK);
}

static u32 dl_xmodem(u32 addr)
{
	u8 ch, *dload;
	long eblock, nblock;
	int err;

	dload  = (void *)addr;
	eblock = 1;
	err    = 0;
        ch     = 'S';
	while(1)
	{
		switch(ch)
		{
		case CAN:	// XMODEM CAN
			err = 0;
			while(1)
			{
				if(getw(&ch, 1000)<0) break;
				else if(ch==CAN) err++;
				if(err>3) return 0;
			}
			break;
		case EOT:	// XMODEM EOF
			putc(ACK);	// ACK
			// Return downloaded size
			return (eblock-1)*128;
		case SOH:	// XMODEM-128  PAK
synced:	// trying to syncronize...
			getw(&ch, 1000);		// [0]:  NBLOCK
			nblock = ch;
			getw(&ch, 1000);		// [1]: ~NBLOCK
			if(nblock==(eblock&0xFF) && ch == (0xFF-nblock))
			{
				int i;
				int crc = 0;

				for(i=0; i<128; i++)
				{
					if(getw(&ch, 1000)<0)
						break;
					crc += ch;
					dload[(eblock-1)*128 + i] = ch;
				}
				if(getw(&ch, 1000)<0)
					err = 1; // blsize
				else if(ch == (crc&0xFF))
					eblock++;
				else
					err = 1;
			}
			else
				err = 1;
			if(!err)
				putc(ACK);
			break;

		case 'S':
			send_nak();
			break;
		default:
			if(getw(&ch, 0)<0) break;	 // Flush
			break;
		}

		if(getw(&ch, 10000)<0)
			send_nak();
		else
			err = 0;
	}	// while(1)
}

u32 entry(env_t *env)
{
	const char *s = "============================================\n";
	u32 addr;
	mbE_t  *i = (void *)ADDR_POST;

	printk("\n");
	printk("=(1)========================================\n");
	printk("            [FW"DATE" POST]\n");
	printk((char *)s);

	//--- Setup environment
	env->warmreset = 0;
	env->romsize = chkROM();
	printk("RAM(%3i Mb) ROM(%3i Kb)\n", env->ramsize>>20, env->romsize>>10);
	printk((char *)s);

	// Scan FLASH signature
	addr = moveImage((u32)i->baseD);

	if(addr!=0xFFFFFFFF)
	{
		if(pci_isHost())
		{
			// Last chance to STOP booting (key pressed)
			wait(500, 1);
			if(keysleft())
				while(keysleft()) getc();
			else
				exit(ADDR_LOAD);	// Jump to download zone
		}
	}

	if(pci_isHost())	 // No signature in Flash, RS upload
	{
		printk("Waiting image (RS/XMODEM)\n");
		while(1)
			if(dl_xmodem(ADDR_LOAD))
			{
				u32 addr;
	
				wait(2000, 0); printk("\n");
				addr = moveImage(ADDR_LOAD);
				if(addr!=0xFFFFFFFF) break;
				printk(	"Bad signature!\n"
					"Waiting image (RS/XMODEM)\n");
			}
		exit(ADDR_LOAD);	// Jump to download zone
	}

	else 		// No signature in daughterboard, PCI upload
	{
		env->warmreset = 1;	// No PCI inicialization, we aren't host
//		printk("Waiting image (PCI/MAILBOX)\n");
		pci_hostInit(NULL);	// Initialize PCI as slave

		exit(ADDR_LOAD);	// Jump to download zone
	}
}
